# -*- coding: utf-8 -*-

# ocitysmap, city map and street index generator from OpenStreetMap data
# Copyright (C) 2020  Hartmut Holzgraefe

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from string import Template
import cairo
import gi
gi.require_version('Rsvg', '2.0')
gi.require_version('Pango', '1.0')
gi.require_version('PangoCairo', '1.0')
from gi.repository import Rsvg, Pango, PangoCairo
import datetime
import locale
import logging
import mapnik
assert mapnik.mapnik_version() >= 300000, \
    "Mapnik module version %s is too old, see ocitysmap's INSTALL " \
    "for more details." % mapnik.mapnik_version_string()
import math
from copy import copy
from gettext import gettext

from ocitysmap.layoutlib import commons
import ocitysmap
from ocitysmap.layoutlib.abstract_renderer import Renderer
from ocitysmap.indexlib.GeneralIndex import GeneralIndexRenderer
from ocitysmap.indexlib.StreetIndex import StreetIndex
from ocitysmap.indexlib.PoiIndex import PoiIndexRenderer, PoiIndex
from indexlib.commons import IndexDoesNotFitError, IndexEmptyError
import ocitysmap.layoutlib.commons as UTILS
import draw_utils
from ocitysmap.maplib.map_canvas import MapCanvas

import time

LOG = logging.getLogger('ocitysmap')


class TK25Renderer(Renderer):
    """
    This Renderer creates a single page map similar to German TK25 
    "Messtischblatt" 
    """

    name = 'tk25'
    description = gettext(u"German TK25 'Messtischblatt' style")

    def __init__(self, db, rc, tmpdir, dpi, file_prefix,
                 index_position = 'side'):

        Renderer.__init__(self, db, rc, tmpdir, dpi)

        self.file_prefix = file_prefix
        
        self._map_coords = self._get_map_coords(None)

        # Prepare the map
        self._map_canvas = self._create_map_canvas(
            float(self._map_coords[2]),  # W
            float(self._map_coords[3]),  # H
            dpi,
            rc.osmid is not None )

        # Commit the internal rendering stack of the map
        self._map_canvas.render()

    def _get_map_coords(self, index_position):
        # TODO: 40x40cm

        x = commons.convert_mm_to_pt(10)
        y = commons.convert_mm_to_pt(10)
        w = commons.convert_mm_to_pt(400)
        h = commons.convert_mm_to_pt(400)

        return (x, y, w, h)

    def _create_index_rendering(self, index_position):
        return None, None

    def _draw_title(self, ctx, w_dots, h_dots, font_face):
        return

    def _draw_copyright_notice(self, ctx, w_dots, h_dots, notice=None,
                               osm_date=None):
        return

    @staticmethod
    def _mm_rect(ctx,x,y,w,h,dpi):
        ctx.rectangle(commons.convert_mm_to_dots(x, dpi),
                      commons.convert_mm_to_dots(y, dpi),
                      commons.convert_mm_to_dots(w, dpi),
                      commons.convert_mm_to_dots(h, dpi))

    @staticmethod
    def _mm_mvto(ctx,x,y,dpi):
        ctx.move_to(commons.convert_mm_to_dots(x, dpi),
                    commons.convert_mm_to_dots(y, dpi))
    @staticmethod
    def _fs(ctx, pt, dpi):
        ctx.set_font_size(pt*dpi/72)

    @staticmethod
    def deg2hm(a):
        (f, d) = math.mpdf(a)
        m = (int)(f*60)
        return d, m

    def render(self, cairo_surface, dpi, osm_date):
        ctx = cairo.Context(cairo_surface)
        pc = PangoCairo.create_context(ctx)

        normal_fd = Pango.FontDescription("DejaVu 7")
        normal_layout, normal_fascent, normal_fheight, normal_em = \
            draw_utils.create_layout_with_font(ctx, normal_fd)

        small_fd  = Pango.FontDescription("DejaVu 4")
        small_layout, small_fascent, small_fheight, small_em = \
            draw_utils.create_layout_with_font(ctx, small_fd)

        PangoCairo.context_set_resolution(normal_layout.get_context(),
                                          96.*dpi/UTILS.PT_PER_INCH)

        # Set a white background
        ctx.save()
        ctx.set_source_rgb(1, 1, 1)
        ctx.rectangle(0, 0, commons.convert_pt_to_dots(self.paper_width_pt, dpi),
                      commons.convert_pt_to_dots(self.paper_height_pt, dpi))
        ctx.fill()
        ctx.restore()

        # Frame around the actual map area
        ctx.save()
        ctx.set_source_rgb(0,0,0)
        ctx.set_line_width(1);
        self._mm_rect(ctx, 40, 40, 40 +  480, 40 + 450, dpi)
        ctx.stroke()
        ctx.restore()

        # Wider outer frame 
        ctx.save()
        ctx.set_source_rgb(0,0,0)
        ctx.set_line_width(5);
        self._mm_rect(ctx, 30, 30, 60 +  480, 60 + 450, dpi)
        ctx.stroke()
        ctx.set_line_width(1);
        self._mm_rect(ctx, 28, 28, 64 +  480, 64 + 450, dpi)
        self._mm_rect(ctx, 32, 32, 56 +  480, 56 + 450, dpi)
        ctx.stroke()
        ctx.restore()

        # coordinates
        ctx.save()
        ctx.set_source_rgb(0,0,0)
        ctx.select_font_face("Droid Sans", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_NORMAL);

        self._mm_mvto(ctx, 33, 42, dpi)
        self._fs(ctx, 7, dpi);
        ctx.show_text("52°");
        self._fs(ctx, 4, dpi);
        ctx.show_text("10'");
        
        self._mm_mvto(ctx, 40, 39, dpi)
        draw_utils.draw_text_left(ctx, normal_layout, normal_fascent,
                                  commons.convert_mm_to_dots(40, dpi), commons.convert_mm_to_dots(39, dpi),
                                  "18°")
        draw_utils.draw_text_right(ctx, normal_layout, normal_fascent,
                                  commons.convert_mm_to_dots(40, dpi), commons.convert_mm_to_dots(41, dpi),
                                  "30'")
        
        ctx.restore()

        cairo_surface.flush()
        return

    @staticmethod
    def _generic_get_minimal_paper_size(bounding_box,
                                        scale=Renderer.DEFAULT_SCALE,
                                        index_position = None):
        return (594, 594) # DinA1 quadratisch

    @staticmethod
    def _sheet2latlng(sheetno):
        yy = sheetno // 100
        xx = sheetno % 100

        lat = (56*60 - yy*6) / 60
        lon = (xx*10 - (5*60 + 40)) / 60

    @staticmethod
    def _latlng2sheet(lat, lon):
        xx = int((lon * 60 - (5*60 + 40)) / 10)
        yy = int((56*60 - lat*60) / 6)

        if yy < 1 or yy > 99:
            return None
        if xx < 0 or xx > 99:
            return None

        sheetno = yy * 100 + xx
        if sheetno not in tk25_names:
            return None
        
        return sheetno

    @staticmethod
    def _sheet2name(sheetno):
        if sheetno in tk25_names:
            return tk25_names[sheetno]
        return none

    @staticmethod
    def get_compatible_paper_sizes(bounding_box,
                                   renderer_context,
                                   scale=Renderer.DEFAULT_SCALE):

        return [
            {
                "name": "Best fit",
                "width": 841,
                "height": 594,
                "portrait_ok": False,
                "landscake_ok": True,
                "default": True,
                "landscape_preferred": True,
            }
        ]

        

     
        
tk25_names = {
    193: "Deutsch Crottingen [Kretingalė]",
    194: "Jakubowo [Jokūbavas]",
    292: "Memel [Klaipėda]",
    293: "Plicken [Plikiai]",
    294: "Laugallen [Laugaliai]",
    392: "Schmelz [Smeltė]",
    393: "Gotzhofen [Žardininkai]",
    394: "Dawillen [Dovilai]",
    492: "Schwarzort [Juodkrantė]",
    493: "Prökuls [Priekulė]",
    494: "Wilkieten [Vilkyčiai]",
    495: "Grünheide [Grynaičiai]",
    519: "Linstrup",
    520: "Schottburg",
    521: "Skudstrup",
    592: "Perwelk [Perwelk]",
    593: "Kinten [Kintai]",
    594: "Kukoreiten [Kukoraičiai]",
    595: "Berzischken [Beržiškiai]",
    619: "Hügum",
    620: "Rödding",
    621: "Jels",
    622: "Christiansfeld",
    623: "Aller",
    692: "Nidden [Nida]",
    693: "Minge [Minė]",
    694: "Heidekrug [Šilutė]",
    695: "Gaidellen [Gaideliai]",
    696: "Wersmeningken [Versmininkai]",
    697: "Ackmonischken [Akmeniškiai]",
    717: "Hvidding-Ufer",
    718: "Hvidding",
    719: "Spandet",
    720: "Gramm",
    721: "Skrydstrup",
    722: "Hadersleben",
    723: "Grarup",
    724: "Aarö",
    791: "Pillkoppen",
    793: "Skirwieth",
    794: "Ruß [Rusnė]",
    795: "Uszlöknen [Užlėkniai]",
    796: "Koadjuthen [Katyčiai]",
    797: "Nattkischken [Natkiškiai]",
    798: "Uszkullmen [Velaičiai]",
    799: "Laugszargen",
    816: "Haffsand",
    817: "Kirkeby",
    818: "Bröns",
    819: "Arrild",
    820: "Toftlund",
    821: "Rauberg",
    822: "Hoptrip",
    823: "Süder Wilstrup",
    824: "Halkerhöft",
    890: "Kunzen",
    891: "Rossitten",
    893: "Kallningken",
    894: "Stucken",
    895: "Kaukehmen",
    896: "Rucken [Залесово]",
    897: "Pogegen [Pagėgiai]",
    898: "Willkischken [Vilkyškiai]",
    899: "Szugken [Žukai]",
    8100: "Leibgirren (Leipgiriai)",
    8101: "Forst Schmalleningken (Smalininkai)",
    916: "List Sylt",
    917: "Jerpstedt",
    918: "Schads",
    919: "Lügumkloster",
    920: "Bedstedt",
    921: "Jordkirch",
    922: "Apenrade",
    923: "Warnitz",
    924: "Norburg",
    925: "Tarup",
    989: "Sarkau [Šarkuva]",
    990: "Mövenhaken",
    993: "Gilge [Gilija]",
    994: "Groß Lappienen",
    995: "Neukirch [Naujoji]",
    996: "Heinrichswalde [Gastos]",
    997: "Tilsit [Tylża]",
    998: "Ragnit [Ragainė]",
    999: "Baltupönen [Baltupėnai]",
    9100: "Wischwill (Viešvilė)",
    9101: "Schmalleningken (Pagėgiai)",
    1015: "Westerland Sylt",
    1016: "Kampen Sylt",
    1017: "Rodenäs",
    1018: "Hoyer",
    1019: "Tondern",
    1020: "Bülderup",
    1021: "Tingleff",
    1022: "Klipleff",
    1023: "Gravenstein",
    1024: "Sonderburg",
    1025: "Augustenburg",
    1026: "Mummark",
    1085: "Groß Dirschkeim [Diržkaimis]",
    1086: "Rauschen [Riaušiai]",
    1087: "Neukuhren [Naujieji Kuršiai]",
    1088: "Cranz [Krantas]",
    1089: "Bledau [Bleduva]",
    1090: "Postnicken West [Paustininkai]",
    1091: "Postnicken Ost",
    1092: "Rinderort [Заливино]",
    1093: "Elchwerder [Nemanynas]",
    1094: "Lauknen [Lauknos]",
    1095: "Grünau [Osseningken]",
    1096: "Jurgaitschen",
    1097: "Schillen [Szillen]",
    1098: "Hohensalzburg [Lengwethen]",
    1099: "Budwethen",
    1115: "Rantum",
    1116: "Morsum Sylt",
    1117: "Horsbüll",
    1118: "Neukirchen",
    1119: "Süderlügum",
    1120: "Ladelund",
    1121: "Medelby",
    1122: "Flensburg Nord",
    1123: "Glücksburg",
    1124: "Westerholz",
    1125: "Kekenis Insel Alsen",
    1126: "Lysabbel",
    1173: "Wittenberg",
    1174: "Dembek",
    1175: "Ostrau",
    1176: "Rixhoft",
    1185: "Palmnicken",
    1186: "Germau",
    1187: "Pobethen",
    1188: "Rudau",
    1189: "Powunden",
    1190: "Schaaken",
    1191: "Lablacken",
    1192: "Labiau",
    1193: "Laukischken",
    1194: "Schargillen [Szargillen]",
    1195: "Groß Skaisgirren",
    1196: "Aulenbach [Aulowönen]",
    1197: "Grünheide",
    1198: "Breitenstein [Kraupischken]",
    1199: "Rautenberg",
    1215: "Hörnum Sylt",
    1216: "Borgsum",
    1217: "Midlum Föhr",
    1218: "Niebüll",
    1219: "Leck",
    1220: "Achtrup",
    1221: "Wallsbüll",
    1222: "Flensburg Süd",
    1223: "Husby",
    1224: "Sterup",
    1225: "Gelting",
    1226: "Öhe",
    1269: "Scholpin",
    1270: "Lebasee",
    1271: "Leba",
    1272: "Sassin",
    1273: "Ossecken",
    1274: "Zarnowitz",
    1275: "Mechau",
    1276: "Putzig",
    1277: "Kußfeld",
    1285: "Lochstadt",
    1286: "Fischhausen",
    1287: "Medenau",
    1288: "Königsberg",
    1289: "Königsberg Ost",
    1290: "Schönwalde",
    1291: "Kaymen",
    1292: "Goldbach",
    1293: "Alt Gertlauken",
    1294: "Groß Schirrau",
    1295: "Escherninken [Eszerninken]",
    1296: "Birken Ostpr. [Neunischken]",
    1297: "Neunischken",
    1298: "Stablacken",
    1299: "Mallwischken",
    1315: "Kniephafen Amrum",
    1316: "Nieblum",
    1317: "Wyk auf Föhr",
    1318: "Ockholm",
    1319: "Bredstedt",
    1320: "Drelsdorf",
    1321: "Jörl",
    1322: "Eggebek",
    1323: "Ülsby",
    1324: "Süderbrarup",
    1325: "Kappeln",
    1326: "Schönhagen",
    1345: "Wiek",
    1346: "Altenkirchen",
    1367: "Neu Strand",
    1368: "Wobesde",
    1369: "Schmolsin",
    1370: "Glowitz",
    1371: "Degendorf",
    1372: "Tauenzin",
    1373: "Saulin",
    1374: "Gohra",
    1375: "Neustadt Westpreußen",
    1376: "Rheda",
    1378: "Hela Befestigungskarte",
    1385: "Pillau",
    1386: "Zimmerbude",
    1387: "Brandenburg (Frisches Haff)",
    1388: "Ponarth",
    1389: "Ludwigswalde",
    1390: "Löwenhagen",
    1391: "Groß Lindenau",
    1392: "Tapiau",
    1393: "Wechlau",
    1394: "Puschdorf",
    1395: "Norkitten",
    1396: "Insterburg",
    1397: "Karalene",
    1398: "Gerwen",
    1399: "Brakuponen",
    1416: "Seesand",
    1417: "Pellworm",
    1418: "Nordstrandischmoor",
    1419: "Wobbenbüll",
    1420: "Hattstedt",
    1421: "Viöl",
    1422: "Jübek",
    1423: "Schleswig",
    1424: "Brodersby",
    1425: "Sieseby",
    1426: "Dorotheenthal",
    1432: "Westermarkelsdorf",
    1433: "Krummensiek",
    1444: "Kloster",
    1445: "Trent",
    1446: "Rappin",
    1447: "Sagard",
    1464: "Vitte",
    1465: "Lanzig",
    1466: "Saleske",
    1467: "Stolpmünde",
    1468: "Freist",
    1469: "Dammen",
    1470: "Stojentin",
    1471: "Schurow",
    1472: "Lauenburg",
    1473: "Roslasin",
    1474: "Lusin",
    1475: "Soppieschin",
    1476: "Kielau",
    1477: "Gotenhafen",
    1484: "Groß Bruch",
    1485: "Balga",
    1486: "Bladiau",
    1487: "Pörschken",
    1488: "Mahnsfeld",
    1489: "Tharau",
    1490: "Uderwangen",
    1491: "Grünbaum",
    1492: "Groß Engelau",
    1493: "Peterswalde",
    1494: "Mulden [Muldszen]",
    1495: "Schulzenhof [Obehlischken]",
    1496: "Jänischen",
    1497: "Sodehnen",
    1498: "Nemmersdorf",
    1499: "Gumbinnen [Гусев]",
    1517: "Süderoog",
    1518: "Südfall",
    1519: "Simonsberg",
    1520: "Husum",
    1521: "Ostenfeld",
    1522: "Hollingstedt",
    1523: "Kropp",
    1524: "Hütten",
    1525: "Eckernförde",
    1526: "Dänischenhagen",
    1527: "Barsbek",
    1528: "Wisch",
    1532: "Petersdorf",
    1533: "Burg (Fehmarn)",
    1540: "Ibenhorst",
    1541: "Prerow",
    1542: "Zingst",
    1543: "Pramort",
    1544: "Klausdorf",
    1545: "Gingst",
    1546: "Bergen auf Rügen",
    1547: "Lubkow",
    1564: "Rügenwalde",
    1565: "Grupenhagen",
    1566: "Peest",
    1567: "Zitzewitz",
    1568: "Stolp",
    1569: "Sageritz",
    1570: "Lupow",
    1571: "Mickrow",
    1572: "Zewitz",
    1573: "Linde",
    1574: "Poblotz",
    1575: "Schönwalde",
    1576: "Quassendorf",
    1577: "Zoppot",
    1583: "Neukrug",
    1584: "Alt Passarge",
    1585: "Heiligenbeil",
    1586: "Deutsch Thierau",
    1587: "Zinten",
    1588: "Kreuzburg",
    1589: "Mühlhausen",
    1590: "Domnau",
    1591: "Georgenau",
    1592: "Friedland (Ostpr.)",
    1593: "Allenburg",
    1594: "Kleingnie",
    1595: "Bokellen",
    1596: "Schwalbental [Jodlauken]",
    1597: "Ballethen",
    1598: "Darkehmen",
    1599: "Praslauken",
    1617: "Ording",
    1618: "Garding",
    1619: "Tönning",
    1620: "Friedrichstadt",
    1621: "Süderstapel",
    1622: "Meggerdorf",
    1623: "Owschlag",
    1624: "Rendsburg",
    1625: "Flemhude",
    1626: "Kiel",
    1627: "Friedrichsort",
    1628: "Bendfeld",
    1629: "Giekau",
    1630: "Hohwacht",
    1631: "Heiligenhafen",
    1632: "Großenbrode",
    1640: "Wustrow",
    1641: "Lüdershagen",
    1642: "Barth",
    1643: "Niepars",
    1644: "Stralsund",
    1645: "Samtens",
    1646: "Putbus",
    1647: "Vilmnitz",
    1648: "Middelhagen",
    1663: "Beelkow",
    1664: "Altenhagen",
    1665: "Karwitz",
    1666: "Schlawe",
    1667: "Kulsow",
    1668: "Rathsdamnitz",
    1669: "Groß Dübsow",
    1670: "Budow",
    1671: "Schwarz Damerkow",
    1672: "Groß Rakitt",
    1673: "Sierke",
    1674: "Chmelno",
    1675: "Karthaus",
    1676: "Zuckau",
    1677: "Danzig",
    1678: "Weichselmünde",
    1679: "Nickelswalde",
    1680: "Steegen",
    1681: "Stutthof",
    1682: "Kahlberg",
    1683: "Tolkemit",
    1684: "Braunsberg West",
    1685: "Braunsberg Ost",
    1686: "Eisenberg",
    1687: "Lichtenfeld",
    1688: "Canditten",
    1689: "Preußisch Eylau",
    1690: "Glommen",
    1691: "Schönbruch",
    1692: "Groß Schönau",
    1693: "Gerdauen",
    1694: "Wandlacken",
    1695: "Nordenburg",
    1696: "Kanitz (Olschöwen)",
    1697: "Groß Sobrost",
    1698: "Almental (Abschermeningken)",
    1699: "Goldap",
    1717: "Böhl",
    1718: "Vollerwiek",
    1719: "Wesselburen",
    1720: "Weddingstedt",
    1721: "Tellingstedt",
    1722: "Dellstedt",
    1723: "Hamdorf Kreis Rendsburg",
    1724: "Bokelholm",
    1725: "Westensee",
    1726: "Flintbek",
    1727: "Preetz",
    1728: "Selent",
    1729: "Lütjenburg",
    1730: "Hansühn",
    1731: "Oldenburg in Holstein",
    1732: "Grube",
    1738: "Fischerbuden",
    1739: "Bad Müritz",
    1740: "Ribnitz",
    1741: "Ahrenshagen",
    1742: "Velgast",
    1743: "Richtenberg",
    1744: "Elmenhorst",
    1745: "Stahlbrode",
    1746: "Zudar",
    1747: "Zickersches Höft",
    1748: "Groß Zicker",
    1749: "Greifswalder Oie",
    1760: "Lassehne",
    1761: "Sorenbohm",
    1762: "Grossmöllen",
    1763: "Zanow",
    1764: "Damerow",
    1765: "Zirchow",
    1766: "Wussow",
    1767: "Varzin",
    1768: "Zuckers",
    1769: "Alt Kolziglow",
    1770: "Wundichow",
    1771: "Jassen",
    1772: "Sullenschin",
    1773: "Stenzitz",
    1774: "Schönberg Westpreußen",
    1775: "Kelpsee [Kelpin]",
    1776: "Prangenau",
    1777: "Praust",
    1778: "Trutenau",
    1779: "Schöneberg",
    1780: "Tiegenhof",
    1781: "Jungfer",
    1782: "Cadinen",
    1783: "Trunz",
    1784: "Tiedmannsdorf",
    1785: "Plasswich",
    1786: "Mehlsack",
    1787: "Lichtenau",
    1788: "Landsberg",
    1789: "Grünwalde",
    1790: "Bartenstein (Ostpr.)",
    1791: "Landskron",
    1792: "Schippenbeil",
    1793: "Dönhofstädt",
    1794: "Barten",
    1795: "Drengfurth",
    1796: "Angerburg",
    1797: "Buddern",
    1798: "Benkheim",
    1799: "Arnswald",
    1813: "Helgoland",
    1818: "Blauort",
    1819: "Büsum",
    1820: "Heide",
    1821: "Nordhastedt",
    1822: "Hanerau-Hademarschen",
    1823: "Todenbüttel",
    1824: "Holtdorf",
    1825: "Nortorf",
    1826: "Bordesholm",
    1827: "Stolpe in Schleswig",
    1828: "Plön",
    1829: "Eutin",
    1830: "Neustadt in Holstein",
    1831: "Grömitz",
    1832: "Klostersee",
    1835: "Rerik",
    1836: "Brunshaupten",
    1837: "Bad Doberan",
    1838: "Warnemünde",
    1839: "Bentwisch",
    1840: "Dänschenburg",
    1841: "Marlow",
    1842: "Drechow",
    1843: "Franzburg",
    1844: "Grimmen",
    1845: "Horst",
    1846: "Neuenkirchen",
    1847: "Wusterhusen",
    1848: "Cröslin",
    1849: "Karlshagen",
    1856: "Kirchhagener Fichten",
    1857: "Robe",
    1858: "Langenhagen",
    1859: "Kolberg",
    1860: "Degow",
    1861: "Kordeshagen",
    1862: "Alt Belz",
    1863: "Köslin",
    1864: "Kösternitz",
    1865: "Alt Zowen",
    1866: "Pollnow",
    1867: "Brotzen",
    1868: "Friedrichshuld",
    1869: "Lubben",
    1870: "Bütow",
    1871: "Lonken",
    1872: "Nakel",
    1873: "Berent",
    1874: "Alt Grabau",
    1875: "Schönfliess Westpreußen",
    1876: "Groß Paglau",
    1877: "Subitz [Sobbowitz]",
    1878: "Mühlbanz",
    1879: "Groß Lichtenau",
    1880: "Neuteich",
    1881: "Zeyer",
    1882: "Elbing",
    1883: "Pomehrendorf",
    1884: "Mühlhausen",
    1885: "Wusen",
    1886: "Wormditt",
    1887: "Migehnen",
    1888: "Raunau",
    1889: "Heilsberg",
    1890: "Gallingen",
    1891: "Groß Schwansfeld",
    1892: "Korschen",
    1893: "Lamgarben",
    1894: "Wenden (Ostpr.)",
    1895: "Rosengarten",
    1896: "Sztynort Wielki",
    1897: "Großgarten",
    1898: "Kerschken",
    1899: "Halldorf",
    1917: "Trischen",
    1919: "Dieksand",
    1920: "Meldorf",
    1921: "Süderhastedt",
    1922: "Schenefeld",
    1923: "Hohenwestedt",
    1924: "Hennstedt",
    1925: "Neumünster",
    1926: "Groß Kummerfeld",
    1927: "Bornhöved",
    1928: "Schlamersdorf",
    1929: "Ahrensbök",
    1930: "Süsel",
    1931: "Rettin",
    1932: "Warnkenhagen",
    1933: "Groß Klütz-Höved",
    1934: "Kaltenhof",
    1935: "Russow",
    1936: "Kröpelin",
    1937: "Hanstorf",
    1938: "Rostock",
    1939: "Petschow",
    1940: "Tessin",
    1941: "Thelkow",
    1942: "Tribsees",
    1943: "Glewitz",
    1944: "Groß Rakow",
    1945: "Griebenow",
    1946: "Greifswald",
    1947: "Hanshagen",
    1948: "Wolgast",
    1949: "Zinnowitz",
    1950: "Ückeritz",
    1954: "Fritzow",
    1955: "Groß Justin",
    1956: "Karnitz",
    1957: "Treptow an der Rega",
    1958: "Gützlaffshagen",
    1959: "Groß Jestin",
    1960: "Kerstin",
    1961: "Belgard",
    1962: "Bulgrin",
    1963: "Seeger",
    1964: "Klannin",
    1965: "Kurow",
    1966: "Sydow",
    1967: "Rummelsburg",
    1968: "Rohr",
    1969: "Kremerbruch",
    1970: "Borczyskowo",
    1971: "Sommin",
    1972: "Kalisch",
    1973: "Funkelkau",
    1974: "Neu Paleschken",
    1975: "Leniewo",
    1976: "Schöneck",
    1977: "Swaroschin",
    1978: "Dirschau",
    1979: "Gnojau",
    1980: "Marienburg",
    1981: "Posilge",
    1982: "Thiergart",
    1983: "Preußisch Holland",
    1984: "Rogehnen",
    1985: "Sommerfeld",
    1986: "Liebstadt",
    1987: "Arnsdorf",
    1988: "Schmolainen",
    1989: "Wernegitten",
    1990: "Frankenau",
    1991: "Bischofstein",
    1992: "Rossel",
    1993: "Heiligelinde",
    1994: "Rastenburg",
    1995: "Sterlawki",
    1996: "Lötzen",
    1997: "Kruglanken",
    1998: "Orlowen",
    1999: "Rogonnen",
    2016: "Neuwerk West",
    2017: "Neuwerk Ost",
    2019: "Kaiser-Wilhelm-Koog",
    2020: "Marne",
    2021: "Burg in Dithmarschen",
    2022: "Wilster",
    2023: "Itzehoe",
    2024: "Kellinghusen",
    2025: "Bad Bramstedt",
    2026: "Heidmühlen",
    2027: "Bad Segeberg",
    2028: "Pronstorf",
    2029: "Curau",
    2030: "Bad Schwartau",
    2031: "Travemünde",
    2032: "Kalkhorst",
    2033: "Hohenkirchen",
    2034: "Kirchdorf",
    2035: "Neuburg",
    2036: "Kirch Mulsow",
    2037: "Satow Niederhagen",
    2038: "Schwaan",
    2039: "Hohen Sprenz",
    2040: "Laage",
    2041: "Walkendorf",
    2042: "Gnoien",
    2043: "Nossendorf",
    2044: "Demmin Pommern",
    2045: "Bentzin",
    2046: "Gützkow",
    2047: "Züssow",
    2048: "Rubkow",
    2049: "Lassan",
    2050: "Benz",
    2051: "Swinemünde",
    2052: "Misdroy",
    2053: "Kolzow",
    2054: "Cammin",
    2055: "Schwirsen",
    2056: "Stuchow",
    2057: "Greifenberg",
    2058: "Kölpin",
    2059: "Roman",
    2060: "Ramelow",
    2061: "Standemin",
    2062: "Boissin",
    2063: "Groß Tychow",
    2064: "Groß Voldekow",
    2065: "Bublitz",
    2066: "Groß Karzenburg",
    2067: "Falkenhagen",
    2068: "Flötenstein",
    2069: "Eisenbrück",
    2070: "Karpno",
    2071: "Rollbick",
    2072: "Lesno",
    2073: "Wille",
    2074: "Wigonin",
    2075: "Hoch Stüblau",
    2076: "Pinschin",
    2077: "Preußisch Stargard",
    2078: "Pelpin",
    2079: "Wernersdorf",
    2080: "Stuhm",
    2081: "Großwaplitz",
    2082: "Christburg",
    2083: "Miswalde",
    2084: "Hagenau",
    2085: "Mohrungen",
    2086: "Reichau",
    2087: "Heiligenthal",
    2088: "Guttstadt",
    2089: "Freudenberg",
    2090: "Seeburg",
    2091: "Rothfließ",
    2092: "Kabienen",
    2093: "Giesewen",
    2094: "Weißenburg",
    2095: "Rhein",
    2096: "Paprodken",
    2097: "Milken",
    2098: "Widminnen",
    2099: "Jucha",
    2117: "Altenwalde",
    2118: "Cuxhaven",
    2119: "Otterndorf",
    2120: "Neuhaus (Oste)",
    2121: "Freiburg Elbe",
    2122: "Krempe",
    2123: "Lägerdorf",
    2124: "Brande-Hörnerkirchen",
    2125: "Kaltenkirchen",
    2126: "Stuvenborn",
    2127: "Leezen",
    2128: "Bad Oldesloe",
    2129: "Hamberge",
    2130: "Lübeck",
    2131: "Schönberg",
    2132: "Mummendorf",
    2133: "Grevesmühlen",
    2134: "Wismar",
    2135: "Zurow",
    2136: "Warin",
    2137: "Bützow",
    2138: "Lüssow",
    2139: "Kritzkow",
    2140: "Schlieffenberg",
    2141: "Thürkow",
    2142: "Neukalen",
    2143: "Verchen",
    2144: "Leistenow",
    2145: "Daberkow",
    2146: "Krien",
    2147: "Medow",
    2148: "Anklam",
    2149: "Usedom",
    2150: "Zirchow",
    2151: "Kaseburg",
    2152: "Lebbin",
    2153: "Wollin",
    2154: "Dobberphul",
    2155: "Gülzow",
    2156: "Schwessow",
    2157: "Plathe an der Rega",
    2158: "Witzmitz",
    2159: "Powalice",
    2160: "Stolzenberg",
    2161: "Arnhausen",
    2162: "Wusterbarth",
    2163: "Groß Krössin",
    2164: "Gramenz",
    2165: "Wurchow",
    2166: "Kasimirshof",
    2167: "Baldenburg",
    2168: "Bölzig",
    2169: "Prechlau",
    2170: "Sampohl",
    2171: "Drzewitz",
    2172: "Bruß",
    2173: "Malachin",
    2174: "Long",
    2175: "Occippel",
    2176: "Lubichow",
    2177: "Bobau",
    2178: "Mewe",
    2179: "Rehhof",
    2180: "Pestlin",
    2181: "Groß Rohdau",
    2182: "Alt Christburg",
    2183: "Saalfeld [Zalewo]",
    2184: "Simnau [Szymonowo]",
    2185: "Sonnenborn [Słonecznik]",
    2186: "Locken [Łukta]",
    2187: "Jonkendorf [Jonkowo]",
    2188: "Göttkendorf [Gutkowo]",
    2189: "Alt Wartenburg [Barczewo]",
    2190: "Wartenburg [Barczewo]",
    2191: "Bischofsburg [Biskupiec]",
    2192: "Bredinken",
    2193: "Sensburg [Mrągowo]",
    2194: "Kossewen [Kosewo]",
    2195: "Nikolaiken",
    2196: "Eichendorf",
    2197: "Arys [Orzysz]",
    2198: "Dippelsee",
    2199: "Grabnick",
    2209: "Norderney",
    2210: "Baltrum",
    2211: "Ostende Langeoog",
    2212: "Spiekeroog",
    2213: "Wangerooge",
    2214: "Alte Mellum",
    2217: "Midlum",
    2218: "Wanna",
    2219: "Ihlienworth",
    2220: "Cadenberge",
    2221: "Hamelwörden",
    2222: "Glückstadt",
    2223: "Elmshorn",
    2224: "Barmstedt",
    2225: "Quickborn Kreis Pinneberg",
    2226: "Wakendorf",
    2227: "Bargteheide",
    2228: "Eichede",
    2229: "Krummesse",
    2230: "Ratzeburg",
    2231: "Carlow",
    2232: "Gadebusch",
    2233: "Mühlen Eichsen",
    2234: "Dambeck",
    2235: "Hohen Viecheln",
    2236: "Brüel",
    2237: "Groß Raden",
    2238: "Zehna",
    2239: "Güstrow",
    2240: "Lalendorf",
    2241: "Teterow",
    2242: "Malchin",
    2243: "Ivenack",
    2244: "Törpin",
    2245: "Gültz",
    2246: "Siedenbollentin",
    2247: "Spantekow",
    2248: "Ducherow",
    2249: "Leopoldshagen",
    2250: "Ueckermünde",
    2251: "Neuwarp",
    2253: "Paulsdorf",
    2254: "Pribbernow",
    2255: "Moratz",
    2256: "Zickerke",
    2257: "Groß Sabow",
    2258: "Regenwalde",
    2259: "Rutzenhagen",
    2260: "Schivelbein",
    2261: "Reinfeld",
    2262: "Bad Polzin",
    2263: "Kollatz",
    2264: "Bärwalde (Pommern)",
    2265: "Persanzig",
    2266: "Neustettin",
    2267: "Demmin",
    2268: "Stegers",
    2269: "Förstenau",
    2270: "Pollnitz",
    2271: "Niesewanz",
    2272: "Rittel",
    2273: "Czersk",
    2274: "Schlachta",
    2275: "Groß Schliewitz",
    2276: "Ossiek",
    2277: "Skurz",
    2278: "Münsterwalde",
    2279: "Marienwerder",
    2280: "Groß Krebs",
    2281: "Riesenburg",
    2282: "Rosenberg",
    2283: "Schwalgendorf (Siemiany)",
    2284: "Schnellwalde (Boręczno)",
    2285: "Osterode (Ostróda)",
    2286: "Thierberg (Zwierzewo)",
    2287: "Dietrichswalde (Gietrzwałd)",
    2288: "Allenstein (Olsztyn)",
    2289: "Groß Kleeberg (Klebark Wielki)",
    2290: "Groß Purden (Bartoły Wielkie)",
    2291: "Mensguth (Dźwierzuty)",
    2292: "Ribben",
    2293: "Aweyden",
    2294: "Peitschendorf",
    2295: "Bubrowko",
    2296: "Sdorren",
    2297: "Wiersbinnen",
    2298: "Seebrücken",
    2299: "Monethen",
    2306: "Borkum",
    2307: "Juist West",
    2308: "Juist East",
    2309: "Hage",
    2310: "Dornum",
    2311: "Esens",
    2312: "Carolinensiel",
    2313: "Hohenkirchen (Oldb.)",
    2314: "Hooksiel",
    2315: "Langwarden",
    2316: "Schmarren",
    2317: "Dorum",
    2318: "Neuenwalde",
    2319: "Bederkesa",
    2320: "Lamstedt",
    2321: "Himmelpforten",
    2322: "Stade",
    2323: "Ütersen",
    2324: "Pinneberg",
    2325: "Niendorf",
    2326: "Fuhlsbüttel",
    2327: "Ahrensburg",
    2328: "Trittau",
    2329: "Nusse",
    2330: "Mölln",
    2331: "Seedorf",
    2332: "Groß Salitz",
    2333: "Groß Brütz",
    2334: "Schwerin",
    2335: "Langen-Brütz",
    2336: "Demen",
    2337: "Borkow",
    2338: "Dobbertin",
    2339: "Krakow",
    2340: "Serrahn",
    2341: "Dahmen",
    2342: "Schwinkendorf",
    2343: "Stavenhagen",
    2344: "Rosenow",
    2345: "Altentreptow",
    2346: "Roga",
    2347: "Friedland (Mecklenburg)",
    2348: "Galenbeck",
    2349: "Ferdinandshof",
    2350: "Eggesin",
    2351: "Rieth (Pommern)",
    2352: "Althagen",
    2353: "Groß Stepenitz",
    2354: "Münchendorf",
    2355: "Basenthin",
    2356: "Naugard",
    2357: "Farbezin",
    2358: "Groß Borckenhagen",
    2359: "Labes",
    2360: "Labenz",
    2361: "Wusterwitz",
    2362: "Neu Wuhrow",
    2363: "Pöhlen",
    2364: "Pielburg",
    2365: "Gellen",
    2366: "Thurow (Pommern)",
    2367: "Hammerstein",
    2368: "Ruthenberg",
    2369: "Christfelde",
    2370: "Schlochau",
    2371: "Konitz",
    2372: "Reetz",
    2373: "Schüttenwalde",
    2374: "Zalesie",
    2375: "Lonsk",
    2376: "Osche",
    2377: "Rinkowken",
    2378: "Neuenburg",
    2379: "Garnsee",
    2380: "Niederzehren",
    2381: "Freystadt (Westpreußen)",
    2382: "Sommerau",
    2383: "Tyngwald",
    2384: "Bergfriede",
    2385: "Seubersdorf",
    2386: "Hirschberg (Ostpreußen)",
    2387: "Elgnowko",
    2388: "Gryzliny",
    2389: "Wuttrienen",
    2390: "Pasym",
    2391: "Grammen",
    2392: "Mingfen",
    2393: "Babienten",
    2394: "Eckertsdorf",
    2395: "Niedersee",
    2396: "Johannisburg",
    2397: "Groß Kessel",
    2398: "Bialla",
    2399: "Großrosen",
    2408: "Westermarsch",
    2409: "Norden",
    2410: "Westerholt",
    2411: "Middels",
    2412: "Wittmund",
    2413: "Jever",
    2414: "Wilhelmshaven",
    2415: "Eckwarden",
    2416: "Stollhamm",
    2417: "Wesermünde-Bremerhaven",
    2418: "Bramel",
    2419: "Köhlen",
    2420: "Ebersdorf",
    2421: "Mulsum",
    2422: "Hagen",
    2423: "Horneburg",
    2424: "Wedel",
    2425: "Hamburg",
    2426: "Wandsbek",
    2427: "Glinde",
    2428: "Schwarzenbek",
    2429: "Siebeneichen",
    2430: "Gudow",
    2431: "Zarrentin",
    2432: "Wittenburg",
    2433: "Parum",
    2434: "Sülstorf",
    2435: "Crivitz",
    2436: "Frauenmark",
    2437: "Herzberg",
    2438: "Goldberg (Mecklenburg)",
    2439: "Karow",
    2440: "Nossentin",
    2441: "Jabel",
    2442: "Waren",
    2443: "Möllenhagen",
    2444: "Penzlin",
    2445: "Neubrandenburg",
    2446: "Pragsdorf",
    2447: "Golm",
    2448: "Strasburg (Uckermark)",
    2449: "Pasewalk",
    2450: "Krugsdorf",
    2451: "Stolzenburg",
    2452: "Falkenwalde",
    2453: "Pölitz",
    2454: "Gollnow",
    2455: "Speck",
    2456: "Eichenwalde",
    2457: "Daber",
    2458: "Silligsdorf",
    2459: "Wangerin",
    2460: "Dramburg",
    2461: "Zülshagen",
    2462: "Falkenburg",
    2463: "Tempelburg",
    2464: "Neblin",
    2465: "Groß Born",
    2466: "Barkenbrügge",
    2467: "Ratzebuhr",
    2468: "Peterswalde",
    2469: "Preußisch Friedland",
    2470: "Grunau",
    2471: "Kamin",
    2472: "Jehlenz",
    2473: "Tuchel",
    2474: "Lindenbusch",
    2475: "Dritschmin",
    2476: "Jeschewo",
    2477: "Warlubien",
    2478: "Feste Courbiere",
    2479: "Roggenhausen",
    2480: "Lessen",
    2481: "Schwenten",
    2482: "Bischofswerder",
    2483: "Deutsch Eylau",
    2484: "Löbau (Westpreußen)",
    2485: "Peterswalde",
    2486: "Geierswalde",
    2487: "Hohenstein",
    2488: "Kurken",
    2489: "Gimmendorf",
    2490: "Gedwangen",
    2491: "Ortelsburg West",
    2492: "Ortelsburg Ost",
    2493: "Schwentainen",
    2494: "Farienen",
    2495: "Mittenheide",
    2496: "Mittel-Pogauen",
    2497: "Morgen",
    2498: "Kolmerfelde",
    2499: "Schwiddern [Świdry]",
    2505: "Usquert",
    2506: "Uithuizermeeden",
    2508: "Pewsum",
    2509: "Loppersum",
    2510: "Aurich (Ostfriesland)",
    2511: "Holtrop",
    2512: "Wiesede",
    2513: "Zetel",
    2514: "Steinhausen",
    2515: "Jadebusen",
    2516: "Nordenham",
    2517: "Loxstedt",
    2518: "Beverstedt",
    2519: "Kirchwistedt",
    2520: "Bremervörde",
    2521: "Bevern",
    2522: "Bargstedt",
    2523: "Harsefeld",
    2524: "Buxtehude",
    2525: "Harburg-Wilhelmsburg",
    2526: "Allermöhe",
    2527: "Bergedorf",
    2528: "Geesthacht",
    2529: "Büchen",
    2530: "Gresse",
    2531: "Camin",
    2532: "Körchow",
    2533: "Hagenow",
    2534: "Lüblow",
    2535: "Hohewisch",
    2536: "Spornitz",
    2537: "Parchim",
    2538: "Lübz",
    2539: "Plau",
    2540: "Malchow",
    2541: "Klink",
    2542: "Federow",
    2543: "Kratzeburg",
    2544: "Hohenzieritz",
    2545: "Burg Stargard (Mecklenburg)",
    2546: "Ballin",
    2547: "Woldegk",
    2548: "Fahrenholz",
    2549: "Nechlin",
    2550: "Brüssow",
    2551: "Löcknitz",
    2552: "Kreckow",
    2553: "Stettin",
    2554: "Groß Christinenberg",
    2555: "Priemhausen",
    2556: "Massow",
    2557: "Schönebeck",
    2558: "Freienwalde (Pommern)",
    2559: "Nörenberg",
    2560: "Zamzow",
    2561: "Großer Lübbesee",
    2562: "Virchow",
    2563: "Groß Linichen",
    2564: "Rederitz",
    2565: "Zippnow",
    2566: "Jastrow",
    2567: "Flederborn",
    2568: "Buschdorf",
    2569: "Linde",
    2570: "Wilhelmsbruch",
    2571: "Zempelburg",
    2572: "Dorfwaldau",
    2573: "Klonowo",
    2574: "Lobfelde (Lubiewo)",
    2575: "Heinrichsdorf (Przysiersk)",
    2576: "Schwetz (Świecie)",
    2577: "Sartowitz",
    2578: "Graudenz (Grudziądz)",
    2579: "Okonin",
    2580: "Kgl. Lindenau (Linowo)",
    2581: "Groß Plowenz (Płowęż)",
    2582: "Skarlin",
    2583: "Neumark",
    2584: "Pronikau (Prątnica)",
    2585: "Elgenau",
    2586: "Gilgenburg (Dąbrówno)",
    2587: "Gardienen",
    2588: "Grünfließ (Napiwoda)",
    2589: "Seehag (Jabłonka)",
    2590: "Malga",
    2591: "Willenberg",
    2592: "Lindenort",
    2593: "Liebenberg",
    2594: "Friedrichshof",
    2595: "Karpen",
    2596: "Wondollen",
    2597: "Gehsen",
    2598: "Lachowo",
    2599: "Lawsk",
    2602: "Anjum",
    2603: "Zoutkamp",
    2605: "Warrfum",
    2606: "Westeremden",
    2607: "Delfzijl",
    2608: "Emden West",
    2609: "Emden",
    2610: "Neermoor",
    2611: "Hesel",
    2612: "Remels",
    2613: "Neuenburg (Oldb.)",
    2614: "Varel",
    2615: "Jaderberg",
    2616: "Brake",
    2617: "Hagen",
    2618: "Hambergen",
    2619: "Kuhstedt",
    2620: "Rhade",
    2621: "Selsingen",
    2622: "Heeslingen",
    2623: "Heidenau",
    2624: "Hollenstedt",
    2625: "Hittfeld",
    2626: "Stelle",
    2627: "Winsen",
    2628: "Artlenburg",
    2629: "Lauenburg (Elbe)",
    2630: "Boizenburg",
    2631: "Besitz",
    2632: "Lübtheen",
    2633: "Redefin",
    2634: "Ludwigslust",
    2635: "Neustadt",
    2636: "Karenzin",
    2637: "Marnitz",
    2638: "Stepenitz",
    2639: "Meyenburg",
    2640: "Dammwolde",
    2641: "Röbel",
    2642: "Rechlin",
    2643: "Blankenförde",
    2644: "Neustrelitz",
    2645: "Thurow (Mecklenburg)",
    2646: "Feldberg",
    2647: "Fürstenwerder",
    2648: "Dedelow",
    2649: "Prenzlau",
    2650: "Wallmow",
    2651: "Hohenholz",
    2652: "Kolbitzow",
    2653: "Podejuch",
    2654: "Altdamm",
    2655: "Kublank",
    2656: "Stargard (Pommern)",
    2657: "Marienfließ",
    2658: "Jacobshagen",
    2659: "Groß Mellen",
    2660: "Rahnwerder",
    2661: "Groß Spiegel",
    2662: "Märk. Friedland",
    2663: "Petznick",
    2664: "Neugolz",
    2665: "Freudenfier",
    2666: "Betkenhammer",
    2667: "Tarnowke",
    2668: "Flatow",
    2669: "Kujan",
    2670: "Vandsburg",
    2671: "Pempersin",
    2672: "Wierzchucyn",
    2673: "Crone",
    2674: "Sirotzken",
    2675: "Grutschno",
    2676: "Culm",
    2677: "Wabcz",
    2678: "Blandau",
    2679: "Rehden",
    2680: "Goßlershausen",
    2681: "Lembe",
    2682: "Pokrzydowo",
    2683: "Kauernik",
    2684: "Mroczno",
    2685: "Rübenau (Rybno)",
    2686: "Usdau",
    2687: "Schläfken",
    2688: "Neidenburg",
    2689: "Muschaken",
    2690: "Roggen",
    2691: "Willenberg Süd",
    2692: "Groß Leschienen",
    2693: "Tscharna",
    2694: "Myschinez",
    2695: "Lipniken",
    2696: "Kuse",
    2697: "Sabjele",
    2698: "Stawiski",
    2699: "Obrytki",
    2702: "Kollum",
    2703: "Grijpskerk",
    2704: "Zuidhorn",
    2705: "Groningen",
    2707: "Siddeburen",
    2708: "Woldendorp",
    2709: "Landschaftspolder",
    2710: "Leer",
    2711: "Nortmoor",
    2712: "Apen",
    2713: "Westerstede",
    2714: "Wiefelstede",
    2715: "Rastede",
    2716: "Elsfleth",
    2717: "Schwanewede",
    2718: "Osterholz-Scharmbeck",
    2719: "Worpswede",
    2720: "Kirchtimke",
    2721: "Zeven",
    2722: "Elsdorf",
    2723: "Groß Sittensen",
    2724: "Tostedt",
    2725: "Holm",
    2726: "Garlstorf",
    2727: "Kirchgellersen",
    2728: "Lüneburg",
    2729: "Neetze",
    2730: "Bleckede",
    2731: "Neuhaus",
    2732: "Jessenitz",
    2733: "Leussow",
    2734: "Eldena",
    2735: "Grabow",
    2736: "Balow",
    2737: "Hülsebeck",
    2738: "Putlitz",
    2739: "Schmolde",
    2740: "Freyenstein",
    2741: "Wredenhagen",
    2742: "Mirow",
    2743: "Wesenberg",
    2744: "Ahrensberg",
    2745: "Lychen",
    2746: "Thomsdorf",
    2747: "Boitzenburg (Uckermark)",
    2748: "Hindenburg in der Mark",
    2749: "Bietikow",
    2750: "Gramzow",
    2751: "Penkun",
    2752: "Greifenhagen",
    2753: "Woltin",
    2754: "Neumark (Pommern)",
    2755: "Werben (Pommern)",
    2756: "Kollin",
    2757: "Zachan",
    2758: "Ravenstein",
    2759: "Reetz",
    2760: "Neuwedell",
    2761: "Kallies",
    2762: "Spechtsdorf",
    2763: "Marzdorf",
    2764: "Deutsch Krone",
    2765: "Breitenstein",
    2766: "Lebehnke",
    2767: "Krojanke",
    2768: "Preussenfeld",
    2769: "Wilhelmshof",
    2770: "Dembno",
    2771: "Mrotschen",
    2772: "Hohenwalde",
    2773: "Wtelno",
    2774: "Zolondowo",
    2775: "Rasmushausen",
    2776: "Unislaw",
    2777: "Papau",
    2778: "Lissewo",
    2779: "Briesen",
    2780: "Wittenburg",
    2781: "Wrotz",
    2782: "Strasburg (Westpreußen)",
    2783: "Radebusch",
    2784: "Lautenburg",
    2785: "Groß Lensk",
    2786: "Heinrichsdorf",
    2787: "Soldau",
    2788: "Bartkenguth",
    2789: "Kirch. Janowetz",
    2790: "Dutschimin Neudorf",
    2791: "Flammberg",
    2792: "Olschewka",
    2793: "Sawady",
    2794: "Kadsidlo",
    2795: "Schafranken",
    2796: "Sbojna",
    2797: "Nowogrod",
    2798: "Czarnocin",
    2799: "Jedwabne",
    2802: "Drachten",
    2803: "Marum",
    2805: "Haren",
    2806: "Hoogezand",
    2807: "Veendam",
    2808: "Winschotem",
    2809: "Bunde",
    2810: "Weener",
    2811: "Westrhauderfehn",
    2812: "Barßel",
    2813: "Edewecht",
    2814: "Bad Zwischenahn",
    2815: "Oldenburg",
    2816: "Berne",
    2817: "Vegesack",
    2818: "Lesum",
    2819: "Lilienthal",
    2820: "Ottersberg",
    2821: "Groẞ Sottrum",
    2822: "Rotenburg (Hannover)",
    2823: "Ostervesede",
    2824: "Schneverdingen",
    2825: "Behringen",
    2826: "Evendorf",
    2827: "Amelinghausen",
    2828: "Bienenbüttel",
    2829: "Altenmedingen",
    2830: "Dahlenburg",
    2831: "Bredenbock",
    2832: "Dannenberg",
    2833: "Dömitz",
    2834: "Gorlosen",
    2835: "Rambow",
    2836: "Karstädt",
    2837: "Bäck",
    2838: "Groß Pankow",
    2839: "Pritzwalk",
    2840: "Wittstock",
    2841: "Babitz",
    2842: "Zechlin",
    2843: "Rheinsberg",
    2844: "Fürstenberg (Mecklenburg)",
    2845: "Himmelpfort",
    2846: "Gandenitz",
    2847: "Templin",
    2848: "Gerswalde",
    2849: "Polssen",
    2850: "Passow",
    2851: "Kunow",
    2852: "Fiddichow",
    2853: "Bahn",
    2854: "Schwochow",
    2855: "Pyritz",
    2856: "Prillwitz",
    2857: "Dölitz",
    2858: "Arnswalde",
    2859: "Sellnow",
    2860: "Fürstenau",
    2861: "Zatten",
    2862: "Tütz",
    2863: "Ruschendorf",
    2864: "Arnsfelde",
    2865: "Groß Wittenberg",
    2866: "Schneidemühl",
    2867: "Schmilau",
    2868: "Wissek",
    2869: "Wirsitz",
    2870: "Sadke",
    2871: "Nakel",
    2872: "Slesin",
    2873: "Bromberg West",
    2874: "Bromberg Ost",
    2875: "Fordon",
    2876: "Birglau",
    2877: "Kulmsee",
    2878: "Kielbasin",
    2879: "Schönsee",
    2880: "Gollub",
    2881: "Hammer",
    2882: "Wilhelmsdank",
    2883: "Gorzberg",
    2884: "Adlig Brinsk",
    2885: "Neuschellen",
    2886: "Königshagen",
    2887: "Illowo",
    2888: "Mlawa",
    2889: "Dzierzgowo",
    2890: "Klein Krynowloga",
    2891: "Ulatowo",
    2892: "Jednoroschetz",
    2893: "Baranow",
    2894: "Dylewo",
    2895: "Wojzechowize",
    2896: "Schkwa",
    2897: "Schtchepankowo Szczepankowo",
    2898: "Lomscha",
    2899: "Bronowo",
    2902: "Beetsterzwaag",
    2903: "Bakkeveen",
    2904: "Veenhuizen",
    2906: "Eext",
    2907: "Nieuwepekela",
    2908: "Onstwedde",
    2909: "Rhede (Ems)",
    2910: "Papenburg",
    2911: "Burlage",
    2912: "Scharrel",
    2913: "Friesoythe",
    2914: "Littel",
    2915: "Wardenburg",
    2916: "Kirchhatten",
    2917: "Delmenhorst",
    2918: "Bremen",
    2919: "Hemelingen",
    2920: "Achim",
    2921: "Ahausen",
    2922: "Kirchwalsede",
    2923: "Wittorf",
    2924: "Neuenkirchen",
    2925: "Bispingen",
    2926: "Breloh",
    2927: "Wriedel",
    2928: "Ebstorf",
    2929: "Bevensen",
    2930: "Himbergen",
    2931: "Gülden",
    2932: "Breese",
    2933: "Groß Gusborn",
    2934: "Lenzen",
    2935: "Schnackenburg",
    2936: "Schilde",
    2937: "Perleberg",
    2938: "Lindenberg",
    2939: "Kolrep",
    2940: "Wutike",
    2941: "Rossow",
    2942: "Zühlen",
    2943: "Dierberg",
    2944: "Gransee",
    2945: "Dannenwalde",
    2946: "Hammelspring",
    2947: "Gollin",
    2948: "Ringenwalde",
    2949: "Greiffenberg (Uckermark)",
    2950: "Angermünde",
    2951: "Schwedt",
    2952: "Uchtdorf",
    2953: "Wildenbruch",
    2954: "Beyersdorf",
    2955: "Lippehne",
    2956: "Schönow",
    2957: "Bernstein",
    2958: "Granow",
    2959: "Schwachenwalde",
    2960: "Bernsee",
    2961: "Regenthin",
    2962: "Schloppe",
    2963: "Klein Drensen",
    2964: "Schönlanke",
    2965: "Radolin",
    2966: "Usch",
    2967: "Morzewo",
    2968: "Samotschin",
    2969: "Jaktorowo",
    2970: "Gromaden",
    2971: "Szaradowo",
    2972: "Schubin",
    2973: "Zielonke",
    2974: "Hopfengarten",
    2975: "Schulitz",
    2976: "Pensau [Pedzewo]",
    2977: "Thorn",
    2978: "Gramtschen",
    2979: "Preußisch Lanke",
    2980: "Neugard",
    2981: "Reddemin",
    2982: "Rypin",
    2983: "Reselerwalde",
    2984: "Sinoberg",
    2985: "Schuromin",
    2986: "Szrensk",
    2987: "Gluscheck",
    2988: "Stupsk",
    2989: "Grudusk",
    2990: "Tschernitz Borowe",
    2991: "Praschnitz",
    2992: "Krasnoselz",
    2993: "Penitz",
    2994: "Nowawes",
    2995: "Scharfenwiese",
    2996: "Troschin",
    2997: "Snjadowo",
    2998: "Batsche",
    2999: "Gaz",
    3002: "Oldeberkoop",
    3003: "Makkinga",
    3004: "Kloosterveen",
    3005: "Assen",
    3006: "Borger",
    3007: "Nieuwbuinen",
    3009: "Dörpen",
    3010: "Wippingen",
    3011: "Börger",
    3012: "Neuarenberg",
    3013: "Markhausen",
    3014: "Garrel",
    3015: "Großenkneten",
    3016: "Dötlingen",
    3017: "Harpstedt",
    3018: "Syke",
    3019: "Riede",
    3020: "Thedinghausen",
    3021: "Verden",
    3022: "Bendingbostel",
    3023: "Visselhövede",
    3024: "Dorfmark",
    3025: "Soltau",
    3026: "Munster",
    3027: "Eimke",
    3028: "Gerdau",
    3029: "Uelzen",
    3030: "Suhlendorf",
    3031: "Klenze",
    3032: "Lüchow",
    3033: "Woltersdorf",
    3034: "Lomitz",
    3035: "Groß Garz",
    3036: "Wittenberge",
    3037: "Bad Wilsnack",
    3038: "Glöwen",
    3039: "Demerthin",
    3040: "Kyritz",
    3041: "Tramnitz",
    3042: "Neuruppin",
    3043: "Lindow",
    3044: "Groß Mutz",
    3045: "Klein Mutz",
    3046: "Zehdenick",
    3047: "Groß Schönebeck (Schorfheide)",
    3048: "Joachimsthal",
    3049: "Groß Ziethen",
    3050: "Stolpe",
    3051: "Zachow",
    3052: "Königsberg (Neumark)",
    3053: "Bad Schönfließ",
    3054: "Schildberg bei Soldin",
    3055: "Soldin",
    3056: "Karzig",
    3057: "Berlinchen",
    3058: "Büssow",
    3059: "Wugarten",
    3060: "Woldenberg",
    3061: "Friedrichsdorf",
    3062: "Eichberg",
    3063: "Groß Drensen",
    3064: "Stieglitz",
    3065: "Scharnikau",
    3066: "Gembitz",
    3067: "Kolmar",
    3068: "Margonin",
    3069: "Gollantsch",
    3070: "Exin",
    3071: "Lindenbruck",
    3072: "Grunhagen",
    3073: "Labischin",
    3074: "Penchowo",
    3075: "Stockrode (Groß Wodek)",
    3076: "Schirpitz",
    3077: "Podgorz",
    3078: "Schillno",
    3079: "Schwarzendorf",
    3080: "Massau",
    3081: "Horstfeld",
    3082: "Ragau",
    3083: "Schuttau",
    3084: "Lutozin",
    3085: "Bieźun",
    3086: "Radsanow",
    3087: "Dombrau",
    3088: "Nedsborsch",
    3090: "Wola-Werchbowska",
    3091: "Bogate",
    3093: "Gasewo",
    3094: "Sielun",
    3095: "Goworowo",
    3096: "Tscherwin",
    3102: "Noordwolde",
    3103: "Vledder",
    3104: "Dwingelo",
    3105: "Beilen",
    3106: "Wezep",
    3107: "Odoorn",
    3108: "Rütenbrock",
    3109: "Lathen",
    3110: "Wahn Han.",
    3111: "Sögel",
    3112: "Werlte",
    3113: "Molbergen",
    3114: "Cloppenburg",
    3115: "Visbek",
    3116: "Wildeshausen",
    3117: "Twistringen",
    3118: "Bassum",
    3119: "Vilsen",
    3120: "Hoya",
    3121: "Dörverden",
    3122: "Kirchboitzen",
    3123: "Walsrode",
    3124: "Fallingbostel",
    3125: "Bergen",
    3126: "Hermannsburg",
    3127: "Unterlüss",
    3128: "Suderburg",
    3129: "Wieren",
    3130: "Bodenteich",
    3131: "Bergen",
    3132: "Salzwedel",
    3133: "Deutsch Pretzier",
    3134: "Arendsee",
    3135: "Leppin",
    3136: "Seehausen",
    3137: "Werben",
    3138: "Havelberg",
    3139: "Lohm",
    3140: "Wusterhausen (Dosse)",
    3141: "Wildberg",
    3142: "Fehrbellin",
    3143: "Wustrau",
    3144: "Beetz",
    3145: "Nassenheide",
    3146: "Liebenwalde",
    3147: "Ruhlsdorf",
    3148: "Eberswalde",
    3149: "Hohenfinow",
    3150: "Oderberg",
    3151: "Zehden",
    3152: "Mohrin",
    3153: "Wartenberg",
    3154: "Rosenthal in der Mark",
    3155: "Staffelde",
    3156: "Fahlenwerder",
    3157: "Lotzen",
    3158: "Altenfliess",
    3159: "Friedeberg (Neumark)",
    3160: "Driesen West",
    3161: "Driesen Ost",
    3162: "Kreuz",
    3163: "Filehne",
    3164: "Gulcz",
    3165: "Lubasch",
    3166: "Schrotthaus",
    3167: "Budsin",
    3168: "Zelice",
    3169: "Wongrowitz",
    3170: "Stempuchowo",
    3171: "Elsenau",
    3172: "Znin",
    3173: "Bartschin",
    3174: "Pakosch",
    3175: "Jakschitz",
    3176: "Argenau",
    3177: "Grabe",
    3178: "Weichselstädt (Alexandrow)",
    3179: "Nessau",
    3180: "Reddemitz",
    3181: "Leipe",
    3182: "Schemmensee",
    3183: "Guisk",
    3184: "Sichelberg",
    3185: "Kirchl. Sawids",
    3186: "Laschewo",
    3187: "Stregow",
    3188: "Oscislau",
    3190: "Pomore",
    3191: "Karnew",
    3192: "Mackeim [Makow]",
    3193: "Czerwonka",
    3195: "Ponikiew",
    3196: "Wonsau",
    3202: "Steenwijk",
    3203: "Havelte",
    3204: "Hoogeveen",
    3205: "Nieuweroord",
    3206: "Oosterhesselen",
    3207: "Noordbarge",
    3208: "Hebelermeer",
    3209: "Haren",
    3210: "Klein Berssen",
    3211: "Holte",
    3212: "Löningen",
    3213: "Essen (Oldb.)",
    3214: "Vestrup",
    3215: "Vechta",
    3216: "Goldenstedt",
    3217: "Barnstorf",
    3218: "Schmalförden",
    3219: "Schwaförden",
    3220: "Bücken (Flecken)",
    3221: "Eystrup",
    3222: "Rethem (Aller)",
    3223: "Ahlden",
    3224: "Ostenholz",
    3225: "Offen",
    3226: "Sülze",
    3227: "Eschede",
    3228: "Sprakensehl",
    3229: "Hankensbüttel",
    3230: "Wittingen",
    3231: "Diesdorf",
    3232: "Beetzendorf",
    3233: "Groß Apenburg",
    3234: "Jeetze",
    3235: "Messdorf",
    3236: "Osterburg",
    3237: "Hindenburg",
    3238: "Sandau",
    3239: "Strohdehne",
    3240: "Rhinow",
    3241: "Friesack",
    3242: "Brunne",
    3243: "Linum",
    3244: "Kremmen",
    3245: "Oranienburg",
    3246: "Wandlitz",
    3247: "Biesenthal",
    3248: "Grüntal (Mark)",
    3249: "Wölsickendorf",
    3250: "Bad Freienwalde",
    3251: "Neulewin",
    3252: "Bärwalde",
    3253: "Fürstenfelde",
    3254: "Neudamm",
    3255: "Massin",
    3256: "Hohenwalde",
    3257: "Landsberg an der Warthe",
    3258: "Zantoch",
    3259: "Lipke",
    3260: "Gottschimm",
    3261: "Altsorge",
    3262: "Schneidemühlchen",
    3263: "Neubrück",
    3264: "Wronke",
    3265: "Obersitzko",
    3266: "Polajewo",
    3267: "Bulowsthal",
    3268: "Rogasen",
    3269: "Markstadt",
    3270: "Janowitz",
    3271: "Rogowo",
    3272: "Gonsawa",
    3273: "Kaisersfelde",
    3274: "Broniewice",
    3275: "Hohensalza",
    3276: "Freitagsheim",
    3277: "Luisenfelde",
    3278: "Koneck",
    3279: "Liebingen",
    3280: "Helmsee",
    3281: "Heiligenbach",
    3282: "Tülchau",
    3283: "Mochau",
    3284: "Lelice",
    3285: "Drobin",
    3286: "Razionsch",
    3287: "Zeschkowo",
    3288: "Kemp [Kepa]",
    3289: "Ojschen",
    3290: "Gonsocin",
    3291: "Porzowo",
    3292: "Ostenburg",
    3293: "Zambski Koscielne",
    3295: "Langsattel",
    3296: "Nagoszewka",
    3297: "Ostrow-Mazowiecka",
    3302: "Genemuiden",
    3303: "Meppel",
    3304: "Kerkenbosch",
    3305: "Slagharen",
    3306: "Coevorden",
    3307: "Emlichheim",
    3308: "Nieuw-Schoonebeek",
    3309: "Meppen",
    3310: "Haselünne",
    3311: "Herzlake",
    3312: "Berge",
    3313: "Quakenbrück",
    3314: "Dinklage",
    3315: "Lohne (Oldenburg)",
    3316: "Diepholz",
    3317: "Barver",
    3318: "Sulingen",
    3319: "Siedenburg",
    3320: "Liebenau (Han.)",
    3321: "Nienburg",
    3322: "Rodewald",
    3323: "Schwarmstedt",
    3324: "Thören",
    3325: "Winsen (Aller)",
    3326: "Celle",
    3327: "Beedenbostel",
    3328: "Groß Oesingen",
    3329: "Wahrenholz",
    3330: "Knesebeck",
    3331: "Brome",
    3332: "Költze West",
    3333: "Klötze East",
    3334: "Calbe",
    3335: "Bismark",
    3336: "Schinne",
    3337: "Stendal",
    3338: "Arneburg",
    3339: "Schollene",
    3340: "Rathenow",
    3341: "Haage",
    3342: "Ribbeck",
    3343: "Nauen",
    3344: "Marwitz",
    3345: "Hennigsdorf",
    3346: "Schönerlinde",
    3347: "Bernau bei Berlin",
    3348: "Werneuchen",
    3349: "Prötzel",
    3350: "Möglin",
    3351: "Neutrebbin",
    3352: "Letschin",
    3353: "Quartschen",
    3354: "Tamsel",
    3355: "Vietz",
    3356: "Klötschen",
    3357: "Dechsel",
    3358: "Trebisch",
    3359: "Schwerin an der Warthe",
    3360: "Waitze",
    3361: "Birnbaum",
    3362: "Zirke",
    3363: "Klodzisko",
    3364: "Scharfenort",
    3365: "Samter",
    3366: "Obornik",
    3367: "Lukowo",
    3368: "Schokken",
    3369: "Jabkowo",
    3370: "Kletzko",
    3371: "Modlin",
    3372: "Goscheschin",
    3373: "Mogilno",
    3374: "Blütenau",
    3375: "Strelno",
    3376: "Chelmce",
    3377: "Radziejow",
    3378: "Ossenholz",
    3379: "Brest",
    3380: "Leslau",
    3381: "Dobrin",
    3382: "Altsiedeln",
    3383: "Sezjen",
    3384: "Belsk",
    3385: "Starozreby",
    3386: "Gora",
    3387: "Baboschew",
    3388: "Plöhnen",
    3389: "Neustadt",
    3390: "Klukow-Golembie",
    3391: "Winnica-(Pultusk)",
    3392: "Poplaw",
    3397: "Sadowne",
    3402: "Zwolle",
    3403: "Dalfsen",
    3404: "Ommen",
    3405: "Hardenberg",
    3406: "Itterbeck",
    3407: "Veldhausen",
    3408: "Wietmarschen",
    3409: "Lingen",
    3410: "Baccum",
    3411: "Lengerich (Han.)",
    3412: "Fürstenau (Han.)",
    3413: "Bersenbrück",
    3414: "Holdorf",
    3415: "Damme",
    3416: "Lembruch",
    3417: "Wagenfeld",
    3418: "Ströhen",
    3419: "Nendorf",
    3420: "Stolzenau",
    3421: "Husum (Kreis Nienburg)",
    3422: "Neustadt am Rbge.",
    3423: "Otternhagen",
    3424: "Mellendorf",
    3425: "Fuhrberg",
    3426: "Wathlingen",
    3427: "Bröckel",
    3428: "Müden",
    3429: "Gamsen",
    3430: "Ehra-Lessien",
    3431: "Steimke",
    3432: "Kunrau",
    3433: "Solpke",
    3434: "Gardelegen",
    3435: "Lindstedt",
    3436: "Lüderitz",
    3437: "Tangermünde",
    3438: "Jerichow",
    3439: "Vieritz",
    3440: "Bamme",
    3441: "Garlitz",
    3442: "Tremmen",
    3443: "Wustermark",
    3444: "Dallgow-Döberitz",
    3445: "Spandau",
    3446: "Berlin Nord",
    3447: "Friedrichsfelde",
    3448: "Alt Landsberg",
    3449: "Strausberg",
    3450: "Müncheberg",
    3451: "Trebnitz",
    3452: "Seelow",
    3453: "Küstrin",
    3454: "Sonnenburg",
    3455: "Alt Limmritz",
    3456: "Kriescht",
    3457: "Königswalde",
    3458: "Blesen",
    3459: "Gollmütz",
    3460: "Prittisch",
    3461: "Kähme",
    3462: "Kwiltsch",
    3463: "Pinne",
    3464: "Ottorowo",
    3465: "Kazmierz",
    3466: "Wargowo",
    3467: "Owinsk",
    3468: "Murowana",
    3469: "Welnau",
    3470: "Libau",
    3471: "Gnesen",
    3472: "Tremessen",
    3473: "Tschemsal",
    3474: "Gembitz",
    3475: "Sierakowo",
    3476: "Strandhof",
    3477: "Piotrkow",
    3478: "Paniewek",
    3479: "Lutbrandau",
    3480: "Kowall",
    3481: "Krebsdorf",
    3482: "Dunau",
    3483: "Sotschewka",
    3484: "Schröttersburg",
    3485: "Radzanowo",
    3486: "Bodsanow",
    3487: "Kusitz",
    3488: "Naruszewo",
    3492: "Serock",
    3493: "Somianka",
    3494: "Wyszkowo",
    3502: "Hattem",
    3503: "Heino",
    3504: "Lemelerveld",
    3505: "Vriezenveen",
    3506: "Getelomoor",
    3507: "Neuenhaus",
    3508: "Nordhorn",
    3509: "Schepsdorf-Lohne",
    3510: "Plantlünne",
    3511: "Freren",
    3512: "Voltlage",
    3513: "Bramsche",
    3514: "Vörden",
    3515: "Hunteburg",
    3516: "Lemförde",
    3517: "Rahden",
    3518: "Diepenau",
    3519: "Uchte",
    3520: "Schlüsselburg",
    3521: "Rehburg",
    3522: "Wunstorf",
    3523: "Stöcken",
    3524: "Isernhagen",
    3525: "Großburgwedel",
    3526: "Burgdorf",
    3527: "Uetze",
    3528: "Meinersen",
    3529: "Gifhorn",
    3530: "Fallersleben",
    3531: "Oebisfelde",
    3532: "Rätzlingen",
    3533: "Mieste",
    3534: "Letzlingen",
    3535: "Dolle",
    3536: "Tangerhütte",
    3537: "Weißewarte",
    3538: "Genthin",
    3539: "Schlagenthin",
    3540: "Plaue",
    3541: "Brandenburg (Havel)",
    3542: "Groß Kreutz",
    3543: "Ketzin",
    3544: "Potsdam Nord",
    3545: "Teltow",
    3546: "Berlin Süd",
    3547: "Köpenick",
    3548: "Rüdersdorf bei Berlin",
    3549: "Herzfelde",
    3550: "Beerfelde",
    3551: "Heinersdorf",
    3552: "Libbenichen",
    3553: "Lebus",
    3554: "Groß Rade",
    3555: "Drossen",
    3556: "Zielenzig",
    3557: "Schermeisel",
    3558: "Tempel",
    3559: "Meseritz",
    3560: "Betsche",
    3561: "Lewitz",
    3562: "Zembowo",
    3563: "Neustadt",
    3564: "Duschnik",
    3565: "Groß Gay",
    3566: "Sady",
    3567: "Posen Nord",
    3568: "Schwersenz",
    3569: "Pudewitz",
    3570: "Schwarzenau",
    3571: "Zydowo",
    3572: "Witkowo",
    3573: "Powidz",
    3574: "Orchowo",
    3575: "Siegersdorf",
    3585: "Miszewo",
    3586: "Wegrzynowo",
    3587: "Rebowo",
    3602: "Wijhe",
    3603: "Raalte",
    3604: "Nijverdal",
    3605: "Almelo",
    3606: "Zenderen",
    3607: "Oldenzaal",
    3608: "Bentheim",
    3609: "Schüttorf",
    3610: "Salzbergen",
    3611: "Hopsten",
    3612: "Mettingen",
    3613: "Westerkappeln",
    3614: "Rulle",
    3615: "Ostercappeln",
    3616: "Levern",
    3617: "Lübbecke",
    3618: "Hartum",
    3619: "Petershagen",
    3620: "Wiedensahl",
    3621: "Stadthagen",
    3622: "Rodenberg",
    3623: "Gehrden",
    3624: "",
    3625: "Lehrte",
    3626: "Haimar",
    3627: "Peine",
    3628: "Wendeburg",
    3629: "Meine",
    3630: "Heiligendorf",
    3631: "Groß Twülpstedt",
    3632: "Weferlingen",
    3633: "Calvörde",
    3634: "Uthmöden",
    3635: "Kolbitz",
    3636: "Rogätz",
    3637: "Parey",
    3638: "Parchen",
    3639: "Karow bei Genthin",
    3640: "Groß Wusterwitz",
    3641: "Göttin",
    3642: "Lehnin",
    3643: "Werder",
    3644: "Potsdam Süd",
    3645: "Großbeeren",
    3646: "Lichtenrade",
    3647: "Königs Wusterhausen",
    3648: "Wernsdorf",
    3649: "Spreenhagen",
    3650: "Fürstenwalde",
    3651: "Briesen in der Mark",
    3652: "Booßen",
    3653: "Frankfurt Oder",
    3654: "Drenzig",
    3655: "Reppen",
    3656: "Sternberg",
    3657: "Lagow",
    3658: "Liebenau",
    3659: "Jordan",
    3660: "Brätz",
    3661: "Tirschtiegel",
    3662: "Neutomischel",
    3663: "Wonsowo",
    3664: "Opalenitza",
    3665: "Buk",
    3666: "Dombrowka",
    3667: "Posen Süd",
    3668: "Gondek",
    3669: "Kostschin",
    3670: "Nekla",
    3671: "Wreschen",
    3672: "Mieltschin",
    3673: "Radlowo",
    3674: "Ostrowite",
    3675: "Lehmstädt (Kleczew)",
    3686: "Ilow",
    3687: "Wyszogrod",
    3692: "Marki",
    3702: "Deventer",
    3703: "Bathmen",
    3704: "Holten",
    3705: "Goor",
    3706: "Hengelo",
    3707: "Enschede",
    3708: "Gronau (Westf.)",
    3709: "Ochtrup",
    3710: "Rheine",
    3711: "Bevergern",
    3712: "Tecklenburg",
    3713: "Hasbergen",
    3714: "Osnabrück",
    3715: "Schledehausen",
    3716: "Melle",
    3717: "Quernheim",
    3718: "Bad Oeynhausen",
    3719: "Minden",
    3720: "Bückeburg",
    3721: "Kathrinhagen",
    3722: "Lauenau",
    3723: "Springe",
    3724: "Pattensen",
    3725: "Sarstedt",
    3726: "Hohenhameln",
    3727: "Groß Ilsede",
    3728: "Vechelde",
    3729: "Braunschweig",
    3730: "Königslutter",
    3731: "Süpplingen",
    3732: "Helmstedt",
    3733: "Erxleben",
    3734: "Haldensleben",
    3735: "Wolmirstedt",
    3736: "Niegripp",
    3737: "Burg",
    3738: "Theessen",
    3739: "Ziesar",
    3740: "Glienecke",
    3741: "Golzow",
    3742: "Damelang",
    3743: "Beelitz",
    3744: "Wildenbruch in der Mark",
    3745: "Trebbin",
    3746: "Zossen",
    3747: "Mittenwalde",
    3748: "Friedersdorf",
    3749: "Storkow",
    3750: "Herzberg",
    3751: "Groß Rietz",
    3752: "Müllrose",
    3753: "Brieskow",
    3754: "Aurith",
    3755: "Groß Gandern",
    3756: "Leichholz",
    3757: "Topper",
    3758: "Mühlbock",
    3759: "Schwiebus",
    3760: "Stentsch",
    3761: "Bentschen",
    3762: "Borui",
    3763: "Konkolewo",
    3764: "Grätz",
    3765: "Granowo",
    3766: "Stenschewo",
    3767: "Moschin",
    3768: "Kurnik",
    3769: "Schroda",
    3770: "Miloslaw",
    3771: "Gozdowo",
    3772: "Sokolnik",
    3773: "Slupca",
    3774: "Golina",
    3775: "Konin",
    3802: "Voorst",
    3806: "Haaksbergen",
    3807: "Alstätte",
    3808: "Nienborg",
    3809: "Metelen",
    3810: "Burgsteinfurt",
    3811: "Emsdetten",
    3812: "Ladbergen",
    3813: "Lengerich Westf.",
    3814: "Iburg",
    3815: "Borgholzhausen",
    3816: "Neuenkirchen",
    3817: "Herford (West)",
    3818: "Herford (Ost)",
    3819: "Vlotho",
    3820: "Rinteln",
    3821: "Hessisch Oldendorf",
    3822: "Hameln",
    3823: "Eldagsen",
    3824: "Elze",
    3825: "Hildesheim",
    3826: "Dingelbe",
    3827: "Lesse",
    3828: "Barum",
    3829: "Wolfenbüttel",
    3830: "Schöppenstedt",
    3831: "Schöningen",
    3832: "Hötensleben",
    3833: "Ummendorf",
    3834: "Groß Rodensleben",
    3835: "Magdeburg",
    3836: "Biederitz",
    3837: "Möckern",
    3838: "Loburg",
    3839: "Altengrabow",
    3840: "Görzke",
    3841: "Belzig",
    3842: "Brück",
    3843: "Buchholz bei Treuenbrietzen",
    3844: "Hennickendorf",
    3845: "Schöneweide",
    3846: "Sperenberg",
    3847: "Teupitz",
    3848: "Märkisch Buchholz",
    3849: "Alt Schadow",
    3850: "Kossenblatt",
    3851: "Beeskow",
    3852: "Grunow",
    3853: "Fünfeichen",
    3854: "Fürstenberg",
    3855: "Tammendorf",
    3856: "Baudach",
    3857: "Beutnitz",
    3858: "Skampe",
    3859: "Kalzig",
    3860: "Bomst",
    3861: "Köbnitz",
    3862: "Wollstein",
    3863: "Rakwitz",
    3864: "Wielichowo",
    3865: "Konojad",
    3866: "Czempin",
    3867: "Brodnica",
    3868: "Santomischel",
    3869: "Sulencin",
    3870: "Pientschkowo",
    3871: "Pogorzelice",
    3872: "Pyzdry",
    3873: "Zagorow",
    3874: "Slawsk",
    3875: "Brezezno",
    3902: "Doesburg",
    3903: "Steenderen",
    3905: "Groenlo",
    3906: "Vreden",
    3907: "Ottenstein",
    3908: "Ahaus",
    3909: "Horstmar",
    3910: "Altenberge",
    3911: "Greven",
    3912: "Westbevern",
    3913: "Ostbevern",
    3914: "Versmold",
    3915: "Bockhorst",
    3916: "Halle Westf.",
    3917: "Bielefeld",
    3918: "Bad Salzuflen",
    3919: "Lemgo",
    3920: "Bösingfeld",
    3921: "Ärzen",
    3922: "Kirchohsen",
    3923: "Salzhemmendorf",
    3924: "Gronau Han.",
    3925: "Sibbesse",
    3926: "Bockenem",
    3927: "Ringelheim",
    3928: "Salzgitter",
    3929: "Hornburg",
    3930: "Hessen",
    3931: "Jerxheim",
    3932: "Hammersleben",
    3933: "Oschersleben",
    3934: "Wanzleben",
    3935: "Groß Ottersleben",
    3936: "Schönebeck",
    3937: "Leitzkau",
    3938: "Lindau (Anhalt)",
    3939: "Nedlitz",
    3940: "Stackelitz",
    3941: "Klepzig",
    3942: "Niemegk",
    3943: "Treuenbrietzen",
    3944: "Zinna",
    3945: "Luckenwalde",
    3946: "Paplitz",
    3947: "Baruth",
    3948: "Oderin",
    3949: "Schlepzig",
    3950: "Groß Leuthen",
    3951: "Niewisch",
    3952: "Groß Muckrow",
    3953: "Neuzelle",
    3954: "Wellmitz",
    3955: "Merzwiese",
    3956: "Crossen",
    3957: "Rädnitz",
    3958: "Rothenburg (Oder)",
    3959: "Züllichau",
    3960: "Trebschen",
    3961: "Unruhstadt",
    3962: "Kiebel",
    3963: "Priment",
    3964: "Poln",
    3965: "Kosten",
    3966: "Choryn",
    3967: "Dalewo",
    3968: "Schrimm",
    3969: "Xions",
    3970: "Neustadt",
    3971: "Zerkow",
    3972: "Robakow",
    3973: "Bialobloty",
    3974: "Grodziec",
    3975: "Tuliszkow",
    4002: "Zevenaar",
    4003: "Doetinchem",
    4004: "Terborg",
    4005: "Aalten",
    4006: "Winterswijk",
    4007: "Stadtlohn",
    4008: "Coesfeld West",
    4009: "Coesfeld Ost",
    4010: "Nottuln",
    4011: "Münster",
    4012: "Telgte",
    4013: "Warendorf",
    4014: "Sassenberg",
    4015: "Harsewinkel",
    4016: "Gütersloh",
    4017: "Brackwede",
    4018: "Lage",
    4019: "Detmold",
    4020: "Blomberg",
    4021: "Bad Pyrmont",
    4022: "Ottenstein",
    4023: "Eschershausen",
    4024: "Alfeld",
    4025: "Groß Freden",
    4026: "Lamspringe",
    4027: "Lutter am Barenberge",
    4028: "Goslar",
    4029: "Vienenburg",
    4030: "Osterwieck",
    4031: "Dardesheim",
    4032: "Schwanebeck",
    4033: "Gröningen",
    4034: "Egeln",
    4035: "Atzendorf",
    4036: "Calbe (Saale)",
    4037: "Barby",
    4038: "Zerbst",
    4039: "Mühlstedt",
    4040: "Hundeluft",
    4041: "Straach",
    4042: "Zahna",
    4043: "Blönsdorf",
    4044: "Jüterbog",
    4045: "Schlenzer",
    4046: "Petkus",
    4047: "Golßen",
    4048: "Waldow",
    4049: "Lübben (Spreewald)",
    4050: "Straupitz",
    4051: "Lieberose",
    4052: "Jamlitz",
    4053: "Grano",
    4054: "Guben",
    4055: "Kanig",
    4056: "Bobersberg",
    4057: "Liebthal",
    4058: "Grünberg West (Niederschles.)",
    4059: "rünberg Ost (Niederschles.)",
    4060: "Fürsteneich",
    4061: "Kontopp",
    4062: "Altkloster",
    4063: "Weine",
    4064: "Luschowitz",
    4065: "Leipe",
    4066: "Storchnest",
    4067: "Lubin",
    4068: "Dolzig",
    4069: "Borek",
    4070: "Gora",
    4071: "Jarotschin",
    4072: "Czernin",
    4073: "Grodzisko",
    4074: "Stawiszyn",
    4075: "Korzeniew",
    4101: "Zyfflich",
    4102: "Elten",
    4103: "Emmerich",
    4104: "Anholt",
    4105: "Bocholt",
    4106: "Rhede (Westf.)",
    4107: "Borken",
    4108: "Groß Reken",
    4109: "Dülmen",
    4110: "Buldern",
    4111: "Ottmarsbocholt",
    4112: "Sendenhorst",
    4113: "Enniger",
    4114: "Oelde",
    4115: "Wiedenbrück",
    4116: "Rietberg",
    4117: "Verl",
    4118: "Senne",
    4119: "Horn",
    4120: "Steinheim",
    4121: "Schwalenberg",
    4122: "Holzminden",
    4123: "Stadtoldendorf",
    4124: "Dassel",
    4125: "Einbeck",
    4126: "Gandersheim",
    4127: "Seesen",
    4128: "Clausthal-Zellerfeld",
    4129: "Bad Harzburg",
    4130: "Wernigerode",
    4131: "Derenburg",
    4132: "Halberstadt",
    4133: "Wegeleben",
    4134: "Cochstedt",
    4135: "Stassfurt",
    4136: "Nienburg",
    4137: "Wulfen",
    4138: "Aken",
    4139: "Dessau",
    4140: "Coswig",
    4141: "Wittenberg Prov. Sachsen",
    4142: "Wartenburg",
    4143: "Seyda",
    4144: "Linda",
    4145: "Schönewalde",
    4146: "Dahme",
    4147: "Uckro",
    4148: "Luckau",
    4149: "Lübbenau (Spreewald)",
    4150: "Burg (Spreewald)",
    4151: "Werben (Spreewald)",
    4152: "Peitz",
    4153: "Strega",
    4154: "Jessnitz",
    4155: "Göhren",
    4156: "Hermswalde",
    4157: "Naumburg an der Bober",
    4158: "Schweinitz Bez. Liegnitz",
    4159: "Günthersdorf",
    4160: "Neusalz",
    4161: "Liebenzig",
    4162: "Schlesiersee [Schlawe]",
    4163: "Fraustadt",
    4164: "Schwetzkau",
    4165: "Lissa",
    4166: "Garzyn",
    4167: "Seide",
    4168: "Gostyn",
    4169: "Pogorzela",
    4170: "Koschmin",
    4171: "Dobrzyca",
    4172: "Pleschen",
    4173: "Tursko",
    4174: "Rychnow",
    4175: "Kamin",
    4201: "Grafwegen",
    4202: "Cleve",
    4203: "Calcar",
    4204: "Rees",
    4205: "Dingden",
    4206: "Brunen",
    4207: "Raesfeld",
    4208: "Wulfen",
    4209: "Haltern",
    4210: "Lündinghausen",
    4211: "Ascheberg",
    4212: "Drensteinfurt",
    4213: "Ahlen",
    4214: "Beckum",
    4215: "Wadersloh",
    4216: "Mastholte",
    4217: "Delbrück",
    4218: "Paderborn",
    4219: "Altenbeken",
    4220: "Bad Driburg",
    4221: "Brakel",
    4222: "Höxter",
    4223: "Sievershausen",
    4224: "Lauenberg",
    4225: "Moringen",
    4226: "Northeim",
    4227: "Osterode",
    4228: "Riefensbeek",
    4229: "St. Andreasberg",
    4230: "Elbingerode",
    4231: "Blankenburg",
    4232: "Quedlinburg",
    4233: "Ballenstedt",
    4234: "Aschersleben",
    4235: "Güsten",
    4236: "Bernburg",
    4237: "Köthen",
    4238: "Quellendorf",
    4239: "Raguhn",
    4240: "Gräfenhainichen",
    4241: "Kemberg",
    4242: "Pretzsch",
    4243: "Jessen",
    4244: "Schweinitz Reg. Bez. Merseburg",
    4245: "Colochau",
    4246: "Schlieben",
    4247: "Walddrehna",
    4248: "Fürstlich Drehna",
    4249: "Calau",
    4250: "Vetschau",
    4251: "Cottbus West",
    4252: "Cottbus Ost",
    4253: "Forst",
    4254: "Pförten",
    4255: "Sommerfeld Lausitz",
    4256: "Gassen",
    4257: "Christianstadt",
    4258: "Briesnitz",
    4259: "Freystadt (Niederschles.)",
    4260: "Beuthen an der Oder",
    4261: "Lindenkranz",
    4262: "Kuttlau",
    4263: "Schlichtingsheim",
    4264: "Seitsch",
    4265: "Lesten",
    4266: "Bojanowo",
    4267: "Kröben",
    4268: "Pempowo",
    4269: "Kobylin",
    4270: "Bozacin",
    4271: "Rozdrazewo",
    4272: "Raschkow",
    4273: "Sobotka",
    4274: "Kalisch",
    4275: "Opatowek",
    4302: "Goch",
    4303: "Uedem",
    4304: "Xanten",
    4305: "Wesel",
    4306: "Drevenack",
    4307: "Dorsten",
    4308: "Marl",
    4309: "Recklinghausen",
    4310: "Waltrop",
    4311: "Lünen",
    4312: "Hamm",
    4313: "Rhynern",
    4314: "Herzfeld",
    4315: "Benninghausen",
    4316: "Lippstadt",
    4317: "Geseke",
    4318: "Etteln",
    4319: "Lichtenau",
    4320: "Willebadessen",
    4321: "Borgholz",
    4322: "Karlshafen",
    4323: "Uslar",
    4324: "Hardegsen",
    4325: "Nörten",
    4326: "Lindau",
    4327: "Gieboldehausen",
    4328: "Bad Lauterberg",
    4329: "Zorge",
    4330: "Benneckenstein",
    4331: "Hasselfelde",
    4332: "Harzgerode",
    4333: "Pansfelde",
    4334: "Leimbach",
    4335: "Hettstedt",
    4336: "Könnern",
    4337: "Löbejün",
    4338: "Zörbig",
    4339: "Bitterfeld West",
    4340: "Bitterfeld Ost",
    4341: "Söllichau",
    4342: "Bad Schmiedeberg",
    4343: "Prettin",
    4344: "Züllsdorf",
    4345: "Herzberg Elster",
    4346: "Buchhain",
    4347: "Kirchhain",
    4348: "Finsterwalde",
    4349: "Göllnitz",
    4350: "Alt Döbern",
    4351: "Drebkau",
    4352: "Komptendorf",
    4353: "Döbern",
    4354: "Triebel",
    4355: "Eichenrode",
    4356: "Sorau",
    4357: "Sagan",
    4358: "Hirschfeldau",
    4359: "Hartau",
    4360: "Neustädtel",
    4361: "Oberquell",
    4362: "Glogau",
    4363: "Weißholz",
    4364: "Schüttlau",
    4365: "Guhrau",
    4366: "Triebusch",
    4367: "Rawitsch",
    4368: "Dubin",
    4369: "Jutroschin",
    4370: "Krotoschin",
    4371: "Sulmirschütz",
    4372: "Ostrowo",
    4373: "Groß Wysotzko",
    4374: "Olobok",
    4375: "Iwanowice",
    4402: "Lindenhof",
    4403: "Geldern",
    4404: "Issum",
    4405: "Rheinberg (Rheinland)",
    4406: "Dinslaken",
    4407: "Bottrop",
    4408: "Gelsenkirchen",
    4409: "Herne",
    4410: "Dortmund",
    4411: "Kamen",
    4412: "Unna",
    4413: "Werl",
    4414: "Soest",
    4415: "Anröchte",
    4416: "Effeln",
    4417: "Büren",
    4418: "Fürstenberg (Westfalen)",
    4419: "Kleinenberg",
    4420: "Peckelsheim",
    4421: "Borgentreich",
    4422: "Trendelburg",
    4423: "Oedelsheim",
    4424: "Dransfeld",
    4425: "Göttingen",
    4426: "Waake",
    4427: "Duderstadt",
    4428: "Weißenborn",
    4429: "Ellrich",
    4430: "Nordhausen Nord",
    4431: "Stolberg",
    4432: "Schwenda",
    4433: "Wippra",
    4434: "Mansfeld",
    4435: "Eisleben",
    4436: "Wettin",
    4437: "Halle Nord",
    4438: "Landsberg bei Halle",
    4439: "Brehna",
    4440: "Delitzsch",
    4441: "Düben",
    4442: "Mockrehna",
    4443: "Torgau West",
    4444: "Torgau Ost",
    4445: "Übigau",
    4446: "Bad Liebenwerda",
    4447: "Oppelhain",
    4448: "Klein Leipisch",
    4449: "Klettwitz",
    4450: "Senftenberg",
    4451: "Jessen Kreis Spremberg",
    4452: "Spremberg",
    4453: "Weißwasser",
    4454: "Muskau",
    4455: "Zibelle",
    4456: "Ndr. Hartmannsdorf",
    4457: "Halbau",
    4458: "Mallmitz",
    4459: "Sprottau",
    4460: "Primkenau",
    4461: "Wiesau",
    4462: "Heerwegen",
    4463: "Raudten",
    4464: "Köben",
    4465: "Gimmel",
    4466: "Herrnstadt",
    4467: "Korsenz",
    4468: "Szkaradowo",
    4469: "Militsch",
    4470: "Preußental",
    4471: "Uciechow",
    4472: "Adelnau",
    4473: "Mixstadt",
    4474: "Grabow a.d. Prosna",
    4475: "Kraszewice",
    4502: "Horst",
    4503: "Straelen",
    4504: "Nieukerk",
    4505: "Moers",
    4506: "Duisburg",
    4507: "Mülheim an der Ruhr",
    4508: "Essen",
    4509: "Bochum",
    4510: "Witten",
    4511: "Hörde",
    4512: "Menden",
    4513: "Neheim",
    4514: "Arnsberg Nord",
    4515: "Hirschberg",
    4516: "Rüthen",
    4517: "Alme",
    4518: "Madfeld",
    4519: "Marsberg",
    4520: "Warburg",
    4521: "Liebenau",
    4522: "Hofgeismar",
    4523: "Hann. Münden",
    4524: "Jühnde",
    4525: "Reinhausen",
    4526: "Gelliehausen",
    4527: "Berlingerode",
    4528: "Worbis",
    4529: "Bleicherode",
    4530: "Nordhausen Süd",
    4531: "Heringen",
    4532: "Kelbra",
    4533: "Sangerhausen",
    4534: "Allstedt",
    4535: "Erdeborn",
    4536: "Schraplau",
    4537: "Halle Süd",
    4538: "Dieskau",
    4539: "Zwochau",
    4540: "Zschortau",
    4541: "Eilenburg",
    4542: "Thallwitz",
    4543: "Schildau",
    4544: "Belgern",
    4545: "Mühlberg",
    4546: "Gröditz",
    4547: "Elsterwerda",
    4548: "Mückenberg",
    4549: "Ruhland",
    4550: "Hohenbocka",
    4551: "Hoyerswerda",
    4552: "Weißkollm",
    4553: "Nochten",
    4554: "Rietschen",
    4555: "Priebus",
    4556: "Freiwaldau",
    4557: "Rauscha",
    4558: "Neuhammer am Queis",
    4559: "Ober Leschen",
    4560: "Armadebrunn",
    4561: "Kotzenau",
    4562: "Gläsersdorf",
    4563: "Koslitz",
    4564: "Steinau (Oder)",
    4565: "Winzig",
    4566: "Groß Bargen",
    4567: "Trachenberg",
    4568: "Friedrichskirch",
    4569: "Katholisch Hammer",
    4570: "Kraschnitz",
    4571: "Tscheschen",
    4572: "Honig",
    4573: "Schildberg",
    4574: "Bukownica",
    4575: "Plugawice",
    4576: "Klonowa",
    4577: "Zloczew",
    4578: "Barczew",
    4602: "Tegelen",
    4603: "Kaldenkirchen",
    4604: "Kempen",
    4605: "Krefeld",
    4606: "Kaiserswerth",
    4607: "Kettwig",
    4608: "Velbert",
    4609: "Hattingen",
    4610: "Hagen",
    4611: "Hohenlimburg",
    4612: "Iserlohn",
    4613: "Balve",
    4614: "Arnsberg Süd",
    4615: "Meschede",
    4616: "Eversberg",
    4617: "Brilon",
    4618: "Adorf",
    4619: "Mengeringhausen",
    4620: "Arolsen",
    4621: "Wolfhagen",
    4622: "Kassel West",
    4623: "Kassel Ost",
    4624: "Hedemünden",
    4625: "Witzenhausen",
    4626: "Heiligenstadt",
    4627: "Dingelstädt",
    4628: "Niederorschel",
    4629: "Keula",
    4630: "Schernberg",
    4631: "Sondershausen",
    4632: "Bad Frankenhausen",
    4633: "Artern",
    4634: "Ziegelroda",
    4635: "Querfurt",
    4636: "Schafstädt",
    4637: "Merseburg West",
    4638: "Merseburg Ost",
    4639: "Leipzig West",
    4640: "Leipzig Ost",
    4641: "Brandis",
    4642: "Wurzen",
    4643: "Dahlen",
    4644: "Wellerswalde-Klingenhain",
    4645: "Riesa",
    4646: "Zeithain",
    4647: "Hirschfeld",
    4648: "Ortrand",
    4649: "Schwepnitz",
    4650: "Bernsdorf",
    4651: "Wittichenau",
    4652: "Lohsa",
    4653: "Spreefurt",
    4654: "Stockteich",
    4655: "Rothenburg (Oberlausitz)",
    4656: "Neuhammer",
    4657: "Tiefenfurt",
    4658: "Klitschdorf",
    4659: "Kittlitztreben",
    4660: "Modlau",
    4661: "Reisicht",
    4662: "Lerchenborn",
    4663: "Lüben",
    4664: "Porschwitz",
    4665: "Wohlau",
    4666: "Stroppen",
    4667: "Prausnitz",
    4668: "Trebnitz (Niederschles.)",
    4669: "Blüchertal",
    4670: "Festenberg",
    4671: "Rudelsdorf",
    4672: "Groß Wartenberg",
    4673: "Rogaszyce",
    4674: "Mikorzyn",
    4675: "Tonia",
    4676: "Lututow",
    4677: "Skrzynno",
    4678: "Hanfhütte",
    4702: "Elmpt",
    4703: "Waldniel",
    4704: "Viersen",
    4705: "Willich",
    4706: "Düsseldorf",
    4707: "Mettmann",
    4708: "Elberfeld Wuppertal",
    4709: "Barmen",
    4710: "Radevormwald",
    4711: "Lüdenscheid",
    4712: "Altena",
    4713: "Plettenberg",
    4714: "Endorf",
    4715: "Eslohe",
    4716: "Bödefeld",
    4717: "Niedersfeld",
    4718: "Goddelsheim",
    4719: "Korbach",
    4720: "Waldeck",
    4721: "Naumburg",
    4722: "Niederzwehren",
    4723: "Oberkaufungen",
    4724: "Großalmerode",
    4725: "Bad Sooden-Allendorf",
    4726: "Grebendorf",
    4727: "Küllstedt",
    4728: "Mühlhausen",
    4729: "Schlotheim",
    4730: "Ebeleben",
    4731: "Greußen",
    4732: "Weißensee",
    4733: "Oberheldrungen",
    4734: "Wiehe",
    4735: "Nebra",
    4736: "Freyburg",
    4737: "Weißenfels",
    4738: "Lützen",
    4739: "Zwenkau",
    4740: "Liebertwolkwitz",
    4741: "Naunhof",
    4742: "Grimma",
    4743: "Mutzschen",
    4744: "Oschatz",
    4745: "Stauchitz",
    4746: "Seußlitz",
    4747: "Großenhain",
    4748: "Radeburg",
    4749: "Königsbrück",
    4750: "Kamenz",
    4751: "Kloster St. Marienstern",
    4752: "Radibor",
    4753: "Weigersdorf",
    4754: "Niesky",
    4755: "Wehrkirch Horka",
    4756: "Penzig",
    4757: "Kohlfurt",
    4758: "Siegersdorf",
    4759: "Bunzlau",
    4760: "Kaiserswaldau",
    4761: "Haynau",
    4762: "Liegnitz",
    4763: "Kunitz",
    4764: "Parchwitz",
    4765: "Regnitz",
    4766: "Dyhernfurth",
    4767: "Auras",
    4768: "Wiese",
    4769: "Perschütz",
    4770: "Oels",
    4771: "Stradam",
    4772: "Trembatschau",
    4773: "Kempen",
    4774: "Baranow",
    4775: "Boleslawice",
    4776: "Biala Rzadowa",
    4777: "Wielun",
    4778: "Ostwerder",
    4802: "Wassenberg",
    4803: "Wegberg",
    4804: "Mönchen-Gladbach",
    4805: "Wevelinghoven",
    4806: "Neuss",
    4807: "Hilden",
    4808: "Solingen",
    4809: "Remscheid",
    4810: "Wipperfürth",
    4811: "Meinerzhagen",
    4812: "Herscheid",
    4813: "Attendorn",
    4814: "Altenhundem",
    4815: "Schmallenberg",
    4816: "Girkhausen",
    4817: "Hallenberg",
    4818: "Medebach",
    4819: "Fürstenberg",
    4820: "Bad Wildungen",
    4821: "Fritzlar",
    4822: "Gudensberg",
    4823: "Melsungen",
    4824: "Hessisch Lichtenau",
    4825: "Waldkappel",
    4826: "Eschwege",
    4827: "Treffurt",
    4828: "Oberdorla",
    4829: "Langensalza",
    4830: "Groß Vargula",
    4831: "Gebesee",
    4832: "Sömmerda",
    4833: "Kölleda",
    4834: "Buttstädt",
    4835: "Eckartsberga",
    4836: "Naumburg",
    4837: "Stössen",
    4838: "Hohenmölsen",
    4839: "Pegau",
    4840: "Borna",
    4841: "Bad Lausick",
    4842: "Colditz",
    4843: "Leisnig",
    4844: "Döbeln",
    4845: "Lommatzsch",
    4846: "Meissen",
    4847: "Radebeul",
    4848: "Klotzsche",
    4849: "Radeberg",
    4850: "Pulsnitz",
    4851: "Bischofswerda",
    4852: "Bautzen",
    4853: "Hochkirch",
    4854: "Reichenbach (Oberlausitz)",
    4855: "Görlitz",
    4856: "Lichtenberg",
    4857: "Lauban",
    4858: "Naumburg am Queis",
    4859: "Löwenberg",
    4860: "Gröditzberg",
    4861: "Goldberg",
    4862: "Kroitsch",
    4863: "Wahlstatt",
    4864: "Groß Tinz",
    4865: "Neumarkt (Niederschlesien)",
    4866: "Leuthen",
    4867: "Breslau-Lissa",
    4868: "Breslau Nord",
    4869: "Sibyllenort",
    4870: "Groß Zöllnig",
    4871: "Bernstadt",
    4872: "Schmograu",
    4873: "Reichthal",
    4874: "Reinersdorf",
    4875: "Pitschen",
    4876: "Wittenau",
    4877: "Dzietrzniki",
    4878: "Kraszkowice",
    4901: "Waldfeucht",
    4902: "Heinsberg",
    4903: "Erkelenz",
    4904: "Titz",
    4905: "Grevenbroich",
    4906: "Stommeln",
    4907: "Hitdorf",
    4908: "Burscheid",
    4909: "Kürten",
    4910: "Lindlar",
    4911: "Gummersbach",
    4912: "Drolshagen",
    4913: "Olpe",
    4914: "Kirchhundem",
    4915: "Wingeshausen",
    4916: "Berleburg",
    4917: "Battenberg",
    4918: "Frankenberg",
    4919: "Frankenau",
    4920: "Armsfeld",
    4921: "Borken",
    4922: "Homberg (Bez. Kassel)",
    4923: "Altmorschen",
    4924: "Seifertshausen",
    4925: "Sontra",
    4926: "Herleshausen",
    4927: "Creuzburg",
    4928: "Mihla",
    4929: "Großenbehringen",
    4930: "Gräfentonna",
    4931: "Walschleben",
    4932: "Stotternheim",
    4933: "Neumark (Thüringen)",
    4934: "Buttelstedt",
    4935: "Apolda",
    4936: "Camburg",
    4937: "Osterfeld",
    4938: "Zeitz",
    4939: "Meuselwitz",
    4940: "Regis-Breitingen",
    4941: "Frohburg",
    4942: "Rochlitz",
    4943: "Geringswalde",
    4944: "Waldheim",
    4945: "Rosswein",
    4946: "Deutschenbora",
    4947: "Wilsdruff",
    4948: "Dresden",
    4949: "Pillnitz",
    4950: "Stolpen",
    4951: "Neustadt",
    4952: "Schirgiswalde",
    4953: "Ebersbach",
    4954: "Löbau",
    4955: "Ostritz",
    4956: "Seidenberg",
    4957: "Marklissa",
    4958: "Greiffenberg (Schlesien)",
    4959: "Liebenthal",
    4960: "Lähn",
    4961: "Schönau an der Katzbach",
    4962: "Kolbnitz",
    4963: "Jauer",
    4964: "Kuhnern",
    4965: "Kostenblut",
    4966: "Kanth",
    4967: "Schmolz",
    4968: "Breslau Süd",
    4969: "Nädlingen",
    4970: "Markstädt",
    4971: "Lampersdorf",
    4972: "Namslau",
    4973: "Noldau",
    4974: "Konstadt",
    4975: "Schönwald",
    4976: "Landsberg",
    4977: "Gollor-Mühle",
    4978: "Lipie",
    5001: "Gangelt",
    5002: "Geilenkirchen",
    5003: "Linnich",
    5004: "Jülich",
    5005: "Bergheim",
    5006: "Frechen",
    5007: "Köln",
    5008: "Mülheim am Rhein",
    5009: "Overath",
    5010: "Engelskirchen",
    5011: "Wiehl",
    5012: "Eckenhagen",
    5013: "Wenden",
    5014: "Hilchenbach",
    5015: "Erndtebrück",
    5016: "Laasphe",
    5017: "Biedenkopf",
    5018: "Wetter Hessen-Nassau",
    5019: "Gemünden",
    5020: "Gilserberg",
    5021: "Ziegenhain",
    5022: "Schwarzenborn",
    5023: "Ludwigseck",
    5024: "Rotenburg an der Fulda",
    5025: "Hönebach",
    5026: "Berka",
    5027: "Eisenach West",
    5028: "Eisenach Ost",
    5029: "Fröttstädt",
    5030: "Gotha",
    5031: "Neudietendorf",
    5032: "Erfurt",
    5033: "Weimar",
    5034: "Magdala",
    5035: "Jena",
    5036: "Bürgel",
    5037: "Eisenberg",
    5038: "Langenberg",
    5039: "Kayna",
    5040: "Altenburg",
    5041: "Langenleuba",
    5042: "Penig",
    5043: "Mittweida",
    5044: "Frankenberg",
    5045: "Langhennersdorf",
    5046: "Freiberg",
    5047: "Freital",
    5048: "Kreischa",
    5049: "Pirna",
    5050: "Königstein",
    5051: "Sebnitz",
    5052: "Großschönau",
    5053: "Warnsdorf",
    5054: "Zittau Nord",
    5055: "Hirschfelde",
    5056: "Weigsdorf",
    5057: "N. Mesto",
    5058: "Friedeberg",
    5059: "Altkemnitz",
    5060: "Hirschberg (Riesengebirge)",
    5061: "Kauffung",
    5062: "Bolkenhain",
    5063: "Hohenfriedeberg",
    5064: "Striegau",
    5065: "Ingramsdorf",
    5066: "Mörschelwitz",
    5067: "Rösslingen",
    5068: "Rothbach",
    5069: "Ohlau",
    5070: "Peisterwitz",
    5071: "Leubusch",
    5072: "Schwirz",
    5073: "Carlsruhe (Oberschlesien)",
    5074: "Bodland",
    5075: "Kreuzburg (Oberschlesien)",
    5076: "Alteneichen",
    5077: "Grunsruh",
    5078: "Krzepice",
    5079: "Klobuck Nord",
    5080: "Ostrowy",
    5081: "Cykarzew",
    5102: "Herzogenrath",
    5103: "Eschweiler",
    5104: "Düren",
    5105: "Buir",
    5106: "Kerpen",
    5107: "Brühl",
    5108: "Wahn",
    5109: "Wahlscheid",
    5110: "Ruppichteroth",
    5111: "Waldbröl",
    5112: "Morsbach",
    5113: "Freudenberg",
    5114: "Siegen",
    5115: "Ewersbach",
    5116: "Eibelshausen",
    5117: "Buchenau",
    5118: "Marburg",
    5119: "Kirchhain Bez. Kassel",
    5120: "Neustadt Kr. Marburg",
    5121: "Schrecksbach",
    5122: "Neukirchen",
    5123: "Niederaula",
    5124: "Bad-Hersfeld",
    5125: "Friedewald",
    5126: "Vacha",
    5127: "Bad-Salzungen",
    5128: "Ruhla",
    5129: "Waltershausen-Friedrichroda",
    5130: "Ohrdruf",
    5131: "Arnstadt",
    5132: "Marlishausen",
    5133: "Kranichfeld",
    5134: "Blankenhain",
    5135: "Kahla",
    5136: "Stadtroda",
    5137: "Münchenbernsdorf",
    5138: "Gera",
    5139: "Ronneburg",
    5140: "Meerane",
    5141: "Glauchau",
    5142: "Hohenstein-Ernstthal",
    5143: "Chemnitz",
    5144: "Flöha",
    5145: "Oederan",
    5146: "Lichtenberg",
    5147: "Frauenstein",
    5148: "Dippoldiswalde",
    5149: "Bad Gottleuba",
    5150: "Rosenthal",
    5151: "Schöna",
    5152: "Böhmisch-Kamnitz",
    5153: "St. Georgenthal",
    5154: "Zittau Süd",
    5155: "Oberullersdorf und Markersdorf",
    5156: "Buschullersdorf",
    5157: "Haindorf",
    5158: "Bad Flinsberg",
    5159: "Schreiberhau",
    5160: "Bad Warmbrunn",
    5161: "Kupferberg",
    5162: "Ruhbank",
    5163: "Freiburg (Niederschles.)",
    5164: "Schweidnitz",
    5165: "Weizenrodau",
    5166: "Zobten",
    5167: "Jordansmühl",
    5168: "Wäldchen",
    5169: "Wansen",
    5170: "Brieg",
    5171: "Stoberau",
    5172: "Alt Poppelau",
    5173: "Kupp",
    5174: "Alt Baudendorf",
    5175: "Sausenberg",
    5176: "Rosenberg",
    5177: "Gnadenkirch",
    5178: "Schlackenufer",
    5179: "Klobuck Süd",
    5180: "Tschenstochau",
    5181: "Mstow",
    5202: "Aachen",
    5203: "Stolberg",
    5204: "Lendersdorf",
    5205: "Vettweiss",
    5206: "Erp",
    5207: "Sechtem",
    5208: "Bonn",
    5209: "Siegburg",
    5210: "Eitorf",
    5211: "Weyerbusch",
    5212: "Wissen",
    5213: "Betzdorf",
    5214: "Burbach",
    5215: "Dillenburg",
    5216: "Oberscheld",
    5217: "Gladenbach",
    5218: "Niederwalgern",
    5219: "Amneburg",
    5220: "Kirtorf",
    5221: "Alsfeld",
    5222: "Grebenbau",
    5223: "Queck",
    5224: "Eiterfeld",
    5225: "Geisa",
    5226: "Stadtlengsfeld",
    5227: "Immelborn",
    5228: "Schmalkalden",
    5229: "Tambach-Dietharz",
    5230: "Gräfenroda",
    5231: "Plaue",
    5232: "Stadtilm",
    5233: "Remda",
    5234: "Rudolstadt",
    5235: "Orlamünde",
    5236: "Neustadt (Thüringen)",
    5237: "Triptis",
    5238: "Weida",
    5239: "Langenbernsdorf",
    5240: "Zwickau",
    5241: "Lichtenstein",
    5242: "Stollberg",
    5243: "Burkhardtsdorf",
    5244: "Zschopau",
    5245: "Lengefeld",
    5246: "Sayda",
    5247: "Nassau",
    5248: "Altenberg",
    5249: "Peterswald",
    5258: "Strickerhäuser",
    5259: "Schneegruben-Baude",
    5260: "Krummhübel",
    5261: "Schmiedeberg (Niederschles.)",
    5262: "Landeshut",
    5263: "Waldenburg",
    5264: "Bad Charlottenbrunn",
    5265: "Reichenbach",
    5266: "Lauterbach",
    5267: "Nimptsch",
    5268: "Strehlen",
    5269: "Marienau",
    5270: "Böhmischdorf",
    5271: "Löwen",
    5272: "Schurgast",
    5273: "Döbern",
    5274: "Königshuld",
    5275: "Radau",
    5276: "Guttentag",
    5277: "Teichwalde",
    5278: "Lissau",
    5279: "Ollschin",
    5280: "Ostatni Grosz",
    5281: "Olsztyn",
    5301: "Herbesthal",
    5302: "Eupen",
    5303: "Roetgen",
    5304: "Nideggen",
    5305: "Zülpich",
    5306: "Euskirchen",
    5307: "Rheinbach",
    5308: "Bad Godesberg",
    5309: "Honnef-Königswinter",
    5310: "Asbach",
    5311: "Altenkirchen",
    5312: "Hachenburg",
    5313: "Marienberg",
    5314: "Rennerod",
    5315: "Herborn",
    5316: "Ballersbach",
    5317: "Rodheim-Bieber",
    5318: "Allendorf",
    5319: "Londorf",
    5320: "Burg-Gemünden",
    5321: "Storndorf",
    5322: "Lauterbach",
    5323: "Schlitz",
    5324: "Hünfeld",
    5325: "Spahl",
    5326: "Tann",
    5327: "Oepfershausen",
    5328: "Wasungen",
    5329: "Zella-Mehlis",
    5330: "Suhl",
    5331: "Ilmenau",
    5332: "Königsee",
    5333: "Blankenburg",
    5334: "Saalfeld",
    5335: "Ziegenrück",
    5336: "Knau",
    5337: "Zeulenroda",
    5338: "Triebes",
    5339: "Greiz",
    5340: "Planitz",
    5341: "Kirchberg",
    5342: "Lössnitz",
    5343: "Geyer",
    5344: "Marienberg",
    5345: "Zöblitz",
    5346: "Olbernhau",
    5347: "Neuwernsdorf",
    5361: "Tschöpsdorf",
    5362: "Schömberg",
    5363: "Friedland Bez. Breslau",
    5364: "Wüstegiersdorf",
    5365: "Langenbielau",
    5366: "Gnadenfrei",
    5367: "Lauenbrunn",
    5368: "Munsterberg",
    5369: "Arnsdorf",
    5370: "Grottkau",
    5371: "Falkenberg",
    5372: "Dambrau",
    5373: "Oppeln",
    5374: "Kranst",
    5375: "Malapane",
    5376: "Grafenweiler [Kolonowskie]",
    5377: "Pawlonkau",
    5378: "Lublininitz",
    5379: "Koschentin",
    5380: "Ellguth",
    5381: "Porai",
    5402: "Ternell",
    5403: "Monschau",
    5404: "Schleiden",
    5405: "Mechernich",
    5406: "Münstereifel",
    5407: "Altenahr",
    5408: "Ahrweiler",
    5409: "Linz",
    5410: "Waldbreitbach",
    5411: "Dierdorf",
    5412: "Selters",
    5413: "Westerburg",
    5414: "Mengerskirchen",
    5415: "Merenberg",
    5416: "Braunfels",
    5417: "Wetzlar",
    5418: "Gießen",
    5419: "Laubach",
    5420: "Schotten",
    5421: "Ulrichstein",
    5422: "Herbstein",
    5423: "Großenlüder",
    5424: "Fulda",
    5425: "Kleinsassen",
    5426: "Hilders",
    5427: "Helmershausen",
    5428: "Meiningen",
    5429: "Themar",
    5430: "Schleusingen",
    5431: "Unterneubrunn",
    5432: "Großbreitenbach",
    5433: "Gräfenthal",
    5434: "Leutenberg",
    5435: "Liebengrün",
    5436: "Schleiz",
    5437: "Pausa - Lössau",
    5438: "Plauen North",
    5439: "Treuen",
    5440: "Auerbach",
    5441: "Schneeberg",
    5442: "Schwarzenberg",
    5443: "Elterlein",
    5444: "Annaberg",
    5445: "Kühnhaide",
    5446: "Görkau",
    5463: "Brunnkress",
    5464: "Wünschelburg",
    5465: "Neurode",
    5466: "Frankenstein",
    5467: "Kamenz (Schlesien)",
    5468: "Lindenau",
    5469: "Bechau",
    5470: "Bösdorf",
    5471: "Tillowitz",
    5472: "Waldfurt",
    5473: "Proskau",
    5474: "Groß Stein",
    5475: "Gr. Strehlitz",
    5476: "Himmelwitz",
    5477: "Keilerswalde",
    5478: "Horneck",
    5479: "Ludwigsthal",
    5480: "Woischnik",
    5481: "Myszkow",
    5501: "Buernenville",
    5502: "Malmedy",
    5503: "Elsenborn",
    5504: "Hellenthal",
    5505: "Blankenheim (Ahr)",
    5506: "Aremberg",
    5507: "Hönningen",
    5508: "Kempenich",
    5509: "Burgbrohl",
    5510: "Neuwied",
    5511: "Bendorf",
    5512: "Montabaur",
    5513: "Meudt",
    5514: "Hadamar",
    5515: "Weilburg",
    5516: "Weilmünster",
    5517: "Cleeberg",
    5518: "Butzbach",
    5519: "Hungen",
    5520: "Nidda",
    5521: "Gedern",
    5522: "Freien-Steinau",
    5523: "Neuhof",
    5524: "Weyhers",
    5525: "Gersfeld",
    5526: "Sondheim (Rhön)",
    5527: "Ostheim",
    5528: "Mühlfeld",
    5529: "Haina",
    5530: "Hildburghausen",
    5531: "Eisfeld",
    5532: "Steinach",
    5533: "Spechtsbrunn (Langenau)",
    5534: "Lehesten",
    5535: "Lobenstein",
    5536: "Hirschberg",
    5537: "Gefell-Münchenreuth",
    5538: "Plauen Süd",
    5539: "Oelsnitz",
    5540: "Falkenstein",
    5541: "Eibenstock",
    5542: "Johanngeorgenstadt",
    5543: "Gottesgab",
    5544: "Hammerunterwiesenthal",
    5563: "Lewin",
    5564: "Bad Reinerz",
    5565: "Glatz",
    5566: "Königshain",
    5567: "Reichenstein",
    5568: "Patschkau",
    5569: "Neisse (West)",
    5570: "Neisse (Ost)",
    5571: "Friedland",
    5572: "Schelitz",
    5573: "Krappitz",
    5574: "Gogolin",
    5575: "Leschnitz",
    5576: "Quellengrund",
    5577: "Tost",
    5578: "Dramatal",
    5579: "Tarnowitz",
    5580: "Brinitz",
    5581: "Siewierz",
    5602: "Recht",
    5603: "Meyerode",
    5604: "Hallschlag",
    5605: "Stadtkyll",
    5606: "Dollendorf",
    5607: "Adenau",
    5608: "Virneburg",
    5609: "Mayen",
    5610: "Bassenheim",
    5611: "Koblenz",
    5612: "Bad Ems",
    5613: "Schaumburg",
    5614: "Limburg",
    5615: "Villmar",
    5616: "Grävenwiesbach",
    5617: "Usingen",
    5618: "Friedberg",
    5619: "Staden",
    5620: "Ortenberg",
    5621: "Wenings",
    5622: "Steinau",
    5623: "Schlüchtern",
    5624: "Brückenau",
    5625: "Wildflecken",
    5626: "Sandberg",
    5627: "Bad Neustadt (Saale)",
    5628: "Mendhausen",
    5629: "Römhild",
    5630: "Rodach bei Coburg",
    5631: "Meeder",
    5632: "Neustadt bei Coburg",
    5633: "Sonneberg",
    5634: "Teuschnitz",
    5635: "Nordhalben",
    5636: "Naila",
    5637: "Hof",
    5638: "Bobenneukirchen",
    5639: "Adorf",
    5640: "Klingenthal",
    5641: "Graslitz",
    5663: "Tassau",
    5664: "Grunwald",
    5665: "Alt Lomnitz",
    5666: "Ullersdorf",
    5667: "Bad Landeck",
    5668: "Ober Hermsdorf",
    5669: "Kaindorf",
    5670: "Ziegenhals",
    5671: "Neustadt",
    5672: "Deutsch Rasselwitz",
    5673: "Oberglogau",
    5674: "Cosel",
    5675: "Heydebreck",
    5676: "Bischofstal",
    5677: "Peiskretscham",
    5678: "Hindenburg",
    5679: "Beuthen",
    5680: "Laurahütte",
    5681: "Dabrowa Gornicza",
    5702: "St. Vith",
    5703: "Bleialf",
    5704: "Prüm",
    5705: "Gerolstein",
    5706: "Hillesheim",
    5707: "Kelberg",
    5708: "Kaisersesch",
    5709: "Kaifenheim",
    5710: "Münstermaifeld",
    5711: "Boppard",
    5712: "Dachsenhausen",
    5713: "Katzenelnbogen",
    5714: "Kettenbach",
    5715: "Idstein",
    5716: "Reifenberg Taunus",
    5717: "Bad Homburg",
    5718: "Ilbenstadt",
    5719: "Altenstadt",
    5720: "Büdingen",
    5721: "Gelnhausen",
    5722: "Salmünster",
    5723: "Altengronau",
    5724: "Zeitlofs",
    5725: "Stangenroth",
    5726: "Bad Kissingen Nord",
    5727: "Münnerstadt",
    5728: "Oberlauringen",
    5729: "Alsleben",
    5730: "Heldburg",
    5731: "Coburg",
    5732: "Sonnefeld",
    5733: "Kronach",
    5734: "Wallenfels",
    5735: "Schwarzenbach am Wald",
    5736: "Helmbrechts",
    5737: "Schwarzenbach (Saale)",
    5738: "Rehau",
    5739: "Bad Elster",
    5740: "Landwüst",
    5764: "Lagenbrück",
    5765: "Habelschwerdt",
    5766: "Wolfelsdorf",
    5767: "Wilhelmsthal",
    5768: "Bielendorf",
    5770: "Arnoldsdorf",
    5771: "Kunzendorf",
    5772: "Leobschutz",
    5773: "Leisnitz",
    5774: "Gr. Neukirch",
    5775: "Reigersfeld",
    5776: "Jakobswalde",
    5777: "Kieferstädtel",
    5778: "Gleiwitz",
    5779: "Schwientochlowitz",
    5780: "Kattowitz",
    5781: "Birkental",
    5801: "Helzingen",
    5802: "Burg Reuland",
    5803: "Leidenborn",
    5804: "Schönecken",
    5805: "Mürlenbach",
    5806: "Daun",
    5807: "Gillenfeld",
    5808: "Cochem",
    5809: "Treis",
    5810: "Dommershausen",
    5811: "Kestert",
    5812: "St. Goarshausen",
    5813: "Nastätten",
    5814: "Bad Schwalbach",
    5815: "Wehen",
    5816: "Königstein (Taunus)",
    5817: "Frankfurt West",
    5818: "Frankfurt Ost",
    5819: "Hanau",
    5820: "Langenselbold",
    5821: "Bieber",
    5822: "Wiesen",
    5823: "Burgsinn",
    5824: "Gräfendorf",
    5825: "Hammelburg",
    5826: "Bad Kissingen Süd",
    5827: "Massbach",
    5828: "Stadtlauringen",
    5829: "Hofheim",
    5830: "Pfarrweisach",
    5831: "Sesslach",
    5832: "Lichtenfels",
    5833: "Burgkunstadt",
    5834: "Kulmbach",
    5835: "Stadtsteinach",
    5836: "Münchberg",
    5837: "Weißenstadt",
    5838: "Selb",
    5839: "Schönberg",
    5865: "Rosenthal",
    5866: "Mittelwalde",
    5871: "Troplowitz",
    5872: "Sauerwitz",
    5873: "Bauerwitz",
    5874: "Preuß. Krawarn",
    5875: "Buchenau",
    5876: "Gr. Rauden",
    5877: "Golleow",
    5878: "Orzesche",
    5879: "Nikolai",
    5880: "Lendzin",
    5881: "Imielin",
    5902: "Dasburg",
    5903: "Neuerburg",
    5904: "Waxweiler",
    5905: "Kyllburg",
    5906: "Manderscheid",
    5907: "Hasborn",
    5908: "Alf",
    5909: "Zell",
    5910: "Kastellaun",
    5911: "Kisselbach",
    5912: "Kaub",
    5913: "Presberg",
    5914: "Eltville",
    5915: "Wiesbaden",
    5916: "Hochheim (Main)",
    5917: "Kelsterbach",
    5918: "Neu Isenburg",
    5919: "Seligenstadt",
    5920: "Alzenau (Unterfr.)",
    5921: "Schöllkrippen",
    5922: "Frammersbach",
    5923: "Rieneck",
    5924: "Gemünden",
    5925: "Gauaschach",
    5926: "Geldersheim",
    5927: "Schweinfurt",
    5928: "Obertheres",
    5929: "Haßfurt",
    5930: "Ebern",
    5931: "Ebensfeld",
    5932: "Ützing",
    5933: "Weismain",
    5934: "Thurnau",
    5935: "Marktschorgast",
    5936: "Bad Berneck",
    5937: "Fichtelberg",
    5938: "Marktredwitz",
    5939: "Waldsassen",
    5940: "Hatzenreuth",
    5972: "Branitz",
    5973: "Altstett",
    5974: "Katscher",
    5975: "Ratibor",
    5976: "Loslau",
    5977: "Rybnik",
    5978: "Sohrau",
    5979: "Kobier",
    5980: "Altberun",
    5981: "Neuberun",
    6002: "Gemünd",
    6003: "Mettendorf",
    6004: "Oberweis",
    6005: "Bitburg",
    6006: "Landscheid",
    6007: "Wittlich",
    6008: "Bernkastel-Kues",
    6009: "Sohren",
    6010: "Kirchberg",
    6011: "Simmern",
    6012: "Stromberg",
    6013: "Bingen",
    6014: "Ingelheim",
    6015: "Mainz",
    6016: "Groß Gerau",
    6017: "Mörfelden",
    6018: "Messel",
    6019: "Babenhausen",
    6020: "Aschaffenburg",
    6021: "Haibach",
    6022: "Rothenbuch",
    6023: "Lohr",
    6024: "Karlstadt",
    6025: "Arnstein",
    6026: "Werneck",
    6027: "Grettstadt",
    6028: "Gerolzhofen",
    6029: "Knetzgau",
    6030: "Eltmann",
    6031: "Bamberg Nord",
    6032: "Scheßlitz",
    6033: "Hollfeld",
    6034: "Mistelgau",
    6035: "Bayreuth",
    6036: "Weidenberg",
    6037: "Ebnath",
    6038: "Waldershof",
    6039: "Mitterteich",
    6040: "Neualbenreuth",
    6041: "Mähring",
    6073: "Troppau",
    6074: "Deutsch Krawarn",
    6075: "Haatsch",
    6076: "Gr. Groschütz",
    6077: "Jastrzemb",
    6078: "Schwarzwasser",
    6079: "Pless",
    6080: "Grzawa",
    6081: "Bielany",
    6102: "Diekirch",
    6103: "Wallendorf",
    6104: "Bollendorf",
    6105: "Welschbillig",
    6106: "Schweich",
    6107: "Neumagen",
    6108: "Morbach",
    6109: "Hottenbach",
    6110: "Gemünden",
    6111: "Pferdsfeld",
    6112: "Waldböckelheim",
    6113: "Bad Kreuznach",
    6114: "Wörrstadt",
    6115: "Undenheim",
    6116: "Oppenheim",
    6117: "Darmstadt West",
    6118: "Darmstadt Ost",
    6119: "Groß Umstadt",
    6120: "Obernburg",
    6121: "Heimbuchenthal",
    6122: "Bischbrunn",
    6123: "Marktheidenfeld",
    6124: "Remlingen",
    6125: "Würzburg",
    6126: "Dettelbach",
    6127: "Volkach",
    6128: "Schallfeld",
    6129: "Burgwindheim",
    6130: "Burgebrach",
    6131: "Bamberg Süd",
    6132: "Buttenheim",
    6133: "Muggendorf",
    6134: "Waischenfeld",
    6135: "Creussen",
    6136: "Kirchenlaibach",
    6137: "Kemnath",
    6138: "Erbendorf",
    6139: "Falkenberg",
    6140: "Tirschenreuth",
    6141: "Treppenstein",
    6173: "Grätz",
    6174: "Schönfeld",
    6175: "Hultschin",
    6176: "Reichwaldau",
    6177: "Freistadt",
    6178: "Petersdorf",
    6179: "Kurzwald",
    6180: "Bielitz",
    6202: "Mersch",
    6203: "Heffingen",
    6204: "Langsur",
    6205: "Trier",
    6206: "Pfalzel",
    6207: "Beuren",
    6208: "Morscheid",
    6209: "Idar-Oberstein",
    6210: "Kirn",
    6211: "Sobernheim",
    6212: "Meisenheim",
    6213: "Kriegsfeld",
    6214: "Mauchenheim",
    6215: "Gau-Odernheim",
    6216: "Gernsheim",
    6217: "Zwingenberg Bergstraße",
    6218: "Neunkirchen",
    6219: "Brensbach",
    6220: "Wörth (Main)",
    6221: "Miltenberg",
    6222: "Stadtprozelten",
    6223: "Wertheim",
    6224: "Helmstadt",
    6225: "Würzburg Süd",
    6226: "Kitzingen",
    6227: "Iphofen",
    6228: "Wiesentheid",
    6229: "Schlüsselfeld",
    6230: "Wachenroth",
    6231: "Willersdorf",
    6232: "Forchheim",
    6233: "Ebermannstadt",
    6234: "Pottenstein",
    6235: "Pegnitz",
    6236: "Eschenbach (Obpf.)",
    6237: "Grafenwöhr",
    6238: "Parkstein",
    6239: "Neustadt Waldnaab",
    6240: "Flossenbürg",
    6273: "Fulnek",
    6274: "Königsberg (Sudentenland)",
    6275: "Gr. Kunzendorf",
    6276: "Bludowitz",
    6277: "Teschen",
    6278: "Golleschau",
    6279: "Brenna",
    6280: "Wolfsdorf",
    6302: "Mamer",
    6303: "Rammeldange",
    6304: "Wincheringen",
    6305: "Saarburg",
    6306: "Kell",
    6307: "Hermeskeil",
    6308: "Birkenfeld West",
    6309: "Birkenfeld Ost",
    6310: "Baumholder",
    6311: "Lauterecken",
    6312: "Rockenhausen",
    6313: "Dannenfels",
    6314: "Kirchheimbolanden",
    6315: "Pfeddersheim",
    6316: "Worms",
    6317: "Bensheim",
    6318: "Lindenfels",
    6319: "Erbach",
    6320: "Michelstadt",
    6321: "Amorbach",
    6322: "Hardheim",
    6323: "Tauberbischofsheim",
    6324: "Grünsfeld",
    6325: "Giebelstadt",
    6326: "Ochsenfurt",
    6327: "Willanzheim",
    6328: "Scheinfeld",
    6329: "Langenfeld",
    6330: "Ühlfeld",
    6331: "Röttenbach",
    6332: "Erlangen Nord",
    6333: "Gräfenberg",
    6334: "Betzenstein",
    6335: "Auerbach",
    6336: "Vilseck",
    6337: "Kaltenbrunn",
    6338: "Weiden",
    6339: "Waldthurn",
    6340: "Vohenstrauß",
    6341: "Frankenreuth",
    6402: "Roeser",
    6403: "Evrange",
    6404: "Kirf",
    6405: "Freudenburg",
    6406: "Losheim",
    6407: "Wadern",
    6408: "Nohfelden",
    6409: "Freisen",
    6410: "Kusel",
    6411: "Wolfstein",
    6412: "Otterberg",
    6413: "Winnweiler",
    6414: "Grünstadt West",
    6415: "Grünstadt Ost",
    6416: "Mannheim Nord",
    6417: "Viernheim",
    6418: "Weinheim",
    6419: "Beerfelden",
    6420: "Schlossau",
    6421: "Buchen",
    6422: "Walldürn",
    6423: "Gissigheim",
    6424: "Lauda",
    6425: "Röttingen",
    6426: "Aub",
    6427: "Uffenheim",
    6428: "Windsheim",
    6429: "Neustadt (Aisch)",
    6430: "Emskirchen",
    6431: "Herzogenaurach",
    6432: "Erlangen Süd",
    6433: "Lauf (Pegnitz)",
    6434: "Hersbruck",
    6435: "Pommelsbrunn",
    6436: "Sulzbach-Rosenberg Nord",
    6437: "Hirschau",
    6438: "Schnaittenbach",
    6439: "Tännesberg",
    6440: "Moosbach",
    6441: "Eslarn",
    6501: "Deutsch Oth",
    6502: "Wollmeringen",
    6503: "Kattenhofen",
    6504: "Sierck",
    6505: "Merzig",
    6506: "Reimsbach",
    6507: "Lebach",
    6508: "Ottweiler",
    6509: "St. Wendel",
    6510: "Glan-Münchweiler",
    6511: "Landstuhl",
    6512: "Kaiserslautern",
    6513: "Hochspeyer",
    6514: "Dürkheim West",
    6515: "Dürkheim Ost",
    6516: "Mannheim Süd",
    6517: "Ladenburg",
    6518: "Heidelberg",
    6519: "Eberbach",
    6520: "Zwingenberg",
    6521: "Oberschefflenz",
    6522: "Adelsheim",
    6523: "Boxberg",
    6524: "Bad Mergentheim",
    6525: "Weikersheim",
    6526: "Creglingen",
    6527: "Burgbernheim",
    6528: "Bergel",
    6529: "Markt Erlbach",
    6530: "Langenzenn",
    6531: "Fürth",
    6532: "Nürnberg",
    6533: "Röthenbach (Pegnitz)",
    6534: "Happurg",
    6535: "Alfeld",
    6536: "Sulzbach-Rosenberg Süd",
    6537: "Amberg",
    6538: "Schmidgaden",
    6539: "Nabburg",
    6540: "Oberviechtach",
    6541: "Tiefenbach",
    6542: "Untergrafenried",
    6601: "Fentsch",
    6602: "Hayingen",
    6603: "Diedenhofen",
    6604: "Monneren",
    6605: "Hemmersdorf",
    6606: "Saarlautern",
    6607: "Heusweiler",
    6608: "Illingen",
    6609: "Neunkirchen",
    6610: "Homburg",
    6611: "Hermersberg",
    6612: "Trippstadt",
    6613: "Elmstein",
    6614: "Neustadt Weinstraße",
    6615: "Haßloch",
    6616: "Speyer",
    6617: "Schwetzingen",
    6618: "Neckargemünd",
    6619: "Epfenbach",
    6620: "Mosbach",
    6621: "Dallau",
    6622: "Möckmühl",
    6623: "Ingelfingen",
    6624: "Dörzbach",
    6625: "Schrozberg",
    6626: "Gammesfeld",
    6627: "Rothenburg ob der Tauber",
    6628: "Leutershausen",
    6629: "Ansbach Nord",
    6630: "Heilsbronn",
    6631: "Roßtal",
    6632: "Schwabach",
    6633: "Feucht",
    6634: "Altdorf",
    6635: "Lauterhofen",
    6636: "Kastl",
    6637: "Rieden",
    6638: "Schwandorf",
    6639: "Wackersdorf",
    6640: "Neunburg vorm Wald",
    6641: "Rötz",
    6642: "Waldmünchen",
    6643: "Furth im Wald",
    6645: "Klattau",
    6701: "Ste Marie",
    6702: "Großmövern",
    6703: "Lüttingen",
    6704: "Gelmingen",
    6705: "Ittersdorf",
    6706: "Ludweiler",
    6707: "Saarbrücken",
    6708: "St. Johann",
    6709: "Blieskastel",
    6710: "Zweibrücken",
    6711: "Pirmasens Nord",
    6712: "Merzalben",
    6713: "Annweiler",
    6714: "Edenkoben",
    6715: "Zeiskam",
    6716: "Germersheim",
    6717: "Wiesental",
    6718: "Wiesloch",
    6719: "Sinsheim",
    6720: "Wimpfen",
    6721: "Kochendorf",
    6722: "Brettach",
    6723: "Öhringen",
    6724: "Künzelsau",
    6725: "Gerabronn",
    6726: "Wiesenbach",
    6727: "Dombühl",
    6728: "Neunstetten",
    6729: "Ansbach Süd",
    6730: "Windsbach",
    6731: "Abenberg",
    6732: "Roth",
    6733: "Allersberg",
    6734: "Neumarkt Oberpfalz",
    6735: "Deining",
    6736: "Velburg",
    6737: "Schmidmühlen",
    6738: "Burglengenfeld",
    6739: "Bruck Oberpfalz",
    6740: "Neukirchen-Balbini",
    6741: "Cham West",
    6742: "Cham Ost",
    6743: "Neukirchen",
    6744: "Neuern",
    6801: "Ste Marie",
    6802: "Gravelotte",
    6803: "Metz",
    6804: "Bolchen",
    6805: "Lubeln",
    6806: "St. Avold",
    6807: "Forbach",
    6808: "Saargemünd",
    6809: "Gersheim",
    6810: "Hornbach",
    6811: "Pirmasens Süd",
    6812: "Dahn",
    6813: "Bergzabern",
    6814: "Landau (Pfalz)",
    6815: "Herxheim",
    6816: "Graben",
    6817: "Bruchsal",
    6818: "Odenheim",
    6819: "Eppingen",
    6820: "Schwaigern",
    6821: "Heilbronn",
    6822: "Willsbach",
    6823: "Pfedelbach",
    6824: "Hall",
    6825: "Ilshofen",
    6826: "Crailsheim",
    6827: "Feuchtwangen West",
    6828: "Feuchtwangen Ost",
    6829: "Ornbau",
    6830: "Gunzenhausen",
    6831: "Spalt",
    6832: "Heideck",
    6833: "Hilpoltstein",
    6834: "Berching",
    6835: "Wissing",
    6836: "Parsberg",
    6837: "Kallmünz",
    6838: "Regenstauf",
    6839: "Nittenau",
    6840: "Reichenbach",
    6841: "Roding",
    6842: "Miltach",
    6843: "Kötzting",
    6844: "Lam",
    6845: "Markt Eisenstein",
    6901: "Gorze",
    6902: "Ars (Mosel)",
    6903: "Werningen",
    6904: "Remilly",
    6905: "Falkenberg (Lothringen)",
    6906: "Ebersingen",
    6907: "Püttlingen",
    6908: "Saaralben",
    6909: "Rohrbach",
    6910: "Bitsch",
    6911: "Ludwigswinkel",
    6912: "Bundenthal",
    6913: "Oberotterbach",
    6914: "Schaidt",
    6915: "Wörth (Rhein)",
    6916: "Karlsruhe",
    6917: "Weingarten",
    6918: "Bretten",
    6919: "Kürnbach",
    6920: "Lauffen",
    6921: "Großbottwar",
    6922: "Löwenstein",
    6923: "Sulzbach (Murr)",
    6924: "Gaildorf",
    6925: "Obersontheim",
    6926: "Jagstheim",
    6927: "Dinkelsbühl",
    6928: "Weiltingen",
    6929: "Wassertrüdingen",
    6930: "Heidenheim",
    6931: "Weißenburg-i-Bay",
    6932: "Nennslingen",
    6933: "Thalmassing",
    6934: "Beilngries",
    6935: "Dietfurt",
    6936: "Hemau",
    6937: "Laaber",
    6938: "Regensburg",
    6939: "Donaustauf",
    6940: "Wörth (Donau)",
    6941: "Stallwang",
    6942: "Englmar",
    6943: "Viechtach",
    6944: "Bodenmais",
    6945: "Zwiesel",
    6946: "Hirschbach",
    7003: "Solgen",
    7004: "Baudrecourt",
    7006: "Groß Tänchen",
    7007: "Insmingen",
    7008: "Saar-Buckenheim",
    7009: "Diemeringen",
    7010: "Saareinsberg",
    7011: "Niederbronn",
    7012: "Wörth",
    7013: "Sulz",
    7014: "Schleital",
    7015: "Durmersheim",
    7016: "Ettlingen",
    7017: "Königsbach",
    7018: "Bauschlott",
    7019: "Vaihingen",
    7020: "Bietigheim",
    7021: "Marbach",
    7022: "Backnang",
    7023: "Murrhardt",
    7024: "Gschwend",
    7025: "Untergröningen",
    7026: "Ellwangen",
    7027: "Zöbingen",
    7028: "Unterschneidheim",
    7029: "Oettingen",
    7030: "Wolferstadt",
    7031: "Treuchtlingen",
    7032: "Bieswang",
    7033: "Titting",
    7034: "Kipfenberg",
    7035: "Schamhaupten",
    7036: "Riedenburg",
    7037: "Kelheim",
    7038: "Bad Abbach",
    7039: "Mintraching",
    7040: "Pfatter",
    7041: "Münster",
    7042: "Bogen",
    7043: "Ruhmannsfelden",
    7044: "Regen",
    7045: "Unterfrauenau",
    7046: "Schwarzach mit Spiegelau",
    7047: "Finsterau",
    7103: "Aulnois",
    7104: "Delmen",
    7105: "Château-Salins",
    7106: "Duß",
    7108: "Finstingen",
    7109: "Lützelstein",
    7110: "Buchsweiler",
    7111: "Pfaffenhofen",
    7112: "Hagenau",
    7113: "Sufflenheim",
    7114: "Selz",
    7115: "Rastatt",
    7116: "Malsch",
    7117: "Neuenbürg",
    7118: "Pforzheim",
    7119: "Weissach",
    7120: "Leonberg",
    7121: "Cannstadt",
    7122: "Winnenden",
    7123: "Schorndorf",
    7124: "Gmünd",
    7125: "Mögglingen",
    7126: "Aalen",
    7127: "Lauchheim",
    7128: "Nördlingen",
    7129: "Deiningen",
    7130: "Wending",
    7131: "Monheim",
    7132: "Dollnstein",
    7133: "Eichstädt",
    7134: "Gaimersheim",
    7135: "Kösching",
    7136: "Neustadt Donau",
    7137: "Abensberg",
    7138: "Langquaid",
    7139: "Aufhausen",
    7140: "Geiselhöring",
    7141: "Straubing",
    7142: "Straßkirchen",
    7143: "Deggendorf",
    7144: "Lalling",
    7145: "Schöfweg",
    7146: "Grafenau",
    7147: "Freyung",
    7148: "Böhmisch-Röhren",
    7204: "Kambrich",
    7206: "Machern",
    7207: "Langenberg",
    7208: "Saarburg",
    7209: "Pfalzburg",
    7210: "Zabern",
    7211: "Hochfelden",
    7212: "Brumath",
    7213: "Scherzheim",
    7214: "Steinbach",
    7215: "Baden-Baden",
    7216: "Gernsbach",
    7217: "Wildbad",
    7218: "Calw",
    7219: "Weil der Stadt",
    7220: "Möhringen",
    7221: "Stuttgart",
    7222: "Plochingen",
    7223: "Göppingen",
    7224: "Lorch",
    7225: "Heubach",
    7226: "Oberkochen",
    7227: "Elchingen",
    7228: "Neresheim",
    7229: "Bissingen",
    7230: "Donauwörth",
    7231: "Genderkingen",
    7232: "Burgheim Nord",
    7233: "Neuburg",
    7234: "Ingolstadt",
    7235: "Vohburg",
    7236: "Münchsmünster",
    7237: "Siegenburg",
    7238: "Rottenburg Laaber",
    7239: "Mallersdorf",
    7240: "Mengkofen",
    7241: "Plisting",
    7242: "Wallersdorf",
    7243: "Plattling",
    7244: "Osterhofen",
    7245: "Schöllnach",
    7246: "Tittling",
    7247: "Waldkirchen",
    7248: "Jandelsbrunn",
    7306: "Elfringen",
    7307: "Rixingen",
    7308: "Albersweiler",
    7310: "Wasselnheim",
    7311: "Truchtersheim",
    7312: "Straßburg Nord",
    7313: "Freistett",
    7314: "Bühl",
    7315: "Bühlertal",
    7316: "Forbach",
    7317: "Simmersfeld",
    7318: "Stammheim",
    7319: "Aidlingen",
    7320: "Böblingen",
    7321: "Neuhausen (Fildern)",
    7322: "Kirchheim unter Teck",
    7323: "Weilheim",
    7324: "Altenstadt",
    7325: "Geislingen (Steige)",
    7326: "Heidenheim",
    7327: "Giengen",
    7328: "Wittislingen",
    7329: "Höchstädt Donau",
    7330: "Mertingen",
    7331: "Rain",
    7332: "Burgheim Süd",
    7333: "Karlshuld",
    7334: "Reichershofen",
    7335: "Geisenfeld",
    7336: "Mainburg",
    7337: "Pfeffenhausen",
    7338: "Hohenthann",
    7339: "Ergoldsbach",
    7340: "Dingolfing West",
    7341: "Dingolfing Ost",
    7342: "Landau (Isar)",
    7343: "Eichendorf",
    7344: "Pleinting",
    7345: "Vilshofen",
    7346: "Hutthurm",
    7347: "Hauzenberg",
    7348: "Wegscheid",
    7408: "Lascemborn",
    7409: "Lützelhausen",
    7410: "Molsheim",
    7411: "Geispolsheim",
    7412: "Kehl",
    7413: "Appenweier",
    7414: "Oberkirch",
    7415: "Obertal",
    7416: "Baiersbronn",
    7417: "Altensteig",
    7418: "Nagold",
    7419: "Herrenberg",
    7420: "Tübingen",
    7421: "Metzingen",
    7422: "Dettingen (Erms)",
    7423: "Wiesensteig",
    7424: "Deggingen",
    7425: "Weidenstetten",
    7426: "Dettingen (Albuch)",
    7427: "Sontheim (Brenz)",
    7428: "Dillingen Donau (West)",
    7429: "Dillingen Donau (Ost)",
    7430: "Wertingen",
    7431: "Thierhaupten",
    7432: "Pöttmes",
    7433: "Schrobenhausen",
    7434: "Hohenwart",
    7435: "Pfaffenhofen (Ilm)",
    7436: "Au in der Hellertau",
    7437: "Bruckberg",
    7438: "Landshut West",
    7439: "Landshut Ost",
    7440: "Aham",
    7441: "Frontenhausen",
    7442: "Arnstorf",
    7443: "Rossbach",
    7444: "Aidenbach",
    7445: "Ortenburg",
    7446: "Passau",
    7447: "Obernzell",
    7448: "Untergriesbach",
    7508: "Blen",
    7509: "Schirmeck",
    7510: "Barr",
    7511: "Erstein",
    7512: "Altenheim",
    7513: "Offenburg",
    7514: "Gengenbach",
    7515: "Bad Peterstal",
    7516: "Freudenstadt",
    7517: "Dornstetten",
    7518: "Horb",
    7519: "Rottenburg",
    7520: "Mössingen",
    7521: "Reutlingen",
    7522: "Urach",
    7523: "Böhringen",
    7524: "Blaubeuren",
    7525: "Ulm Nordwest",
    7526: "Ulm Nordost",
    7527: "Günzburg",
    7528: "Burgau",
    7529: "Zusmarshausen",
    7530: "Gablingen",
    7531: "Gersthofen",
    7532: "Aichach",
    7533: "Kühbach",
    7534: "Petershausen",
    7535: "Allershausen",
    7536: "Freising Nord",
    7537: "Mossburg",
    7538: "Buch am Erlbach",
    7539: "Geisenhausen",
    7540: "Vilsbiburg",
    7541: "Gangkofen",
    7542: "Eggenfelden",
    7543: "Pfarrkirchen",
    7544: "Birnbach",
    7545: "Griesbach",
    7546: "Neuhaus (Inn)",
    7608: "Saal",
    7609: "Weiler",
    7610: "Dambach",
    7611: "Benfeld",
    7612: "Ottenheim",
    7613: "Lahr",
    7614: "Zell",
    7615: "Oberwolfach",
    7616: "Alpirsbach",
    7617: "Sulz",
    7618: "Binsdorf",
    7619: "Hechingen",
    7620: "Jungingen",
    7621: "Undingen",
    7622: "Buttenhausen",
    7623: "Mehrstetten",
    7624: "Schelklingen",
    7625: "Ulm Südwest",
    7626: "Unterkirchberg",
    7627: "Ichenhausen",
    7628: "Jettingen",
    7629: "Dinkelscherben",
    7630: "Westheim",
    7631: "Augsburg",
    7632: "Dasing",
    7633: "Altomünster",
    7634: "Markt Indersdorf",
    7635: "Haimhausen",
    7636: "Freising Süd",
    7637: "Erding",
    7638: "Taufkirchen (Vils)",
    7639: "Velden",
    7640: "Egglkofen",
    7641: "Neumarkt St Veit",
    7642: "Wurmannsquick",
    7643: "Tann",
    7644: "Triftern",
    7645: "Rotthalmünster",
    7646: "Würding",
    7708: "Eckkirch",
    7709: "Markirch",
    7710: "Schlettstadt",
    7711: "Weisweil",
    7712: "Ettenheim",
    7713: "Schweighausen",
    7714: "Haslach",
    7715: "Hornberg",
    7716: "Schramberg",
    7717: "Oberndorf",
    7718: "Geislingen",
    7719: "Balingen",
    7720: "Ebingen",
    7721: "Mägerkingen",
    7722: "Zwiefalten",
    7723: "Munderkingen",
    7724: "Ehingen",
    7725: "Laupheim",
    7726: "Illertissen",
    7727: "Buch",
    7728: "Krumbach (Schwaben)",
    7729: "Ziemetshausen",
    7730: "Großaitingen",
    7731: "Mering",
    7732: "Mammendorf",
    7733: "Maisach",
    7734: "Dachau",
    7735: "Oberschleißheim",
    7736: "Ismaning",
    7737: "Altenerding",
    7738: "Dorfen",
    7739: "Schwindegg",
    7740: "Ampfing",
    7741: "Mühldorf Inn",
    7742: "Altötting",
    7743: "Marktl",
    7744: "Simbach Inn",
    7808: "Urbeis",
    7809: "Rappoltsweiler",
    7810: "Gemar",
    7811: "Sasbach",
    7812: "Endingen",
    7813: "Emmendingen",
    7814: "Elzach",
    7815: "Triberg",
    7816: "Königsfeld",
    7817: "Rottweil",
    7818: "Wehingen",
    7819: "Messtetten",
    7820: "Winterlingen",
    7821: "Veringenstadt",
    7822: "Riedlingen",
    7823: "Uttenweiler",
    7824: "Warthausen",
    7825: "Schwendi",
    7826: "Kirchberg",
    7827: "Babenhausen",
    7828: "Kirchheim in Schwaben",
    7829: "Ettringen",
    7830: "Schwabmünchen",
    7831: "Egling (Paar)",
    7832: "Türkenfeld",
    7833: "Fürstenfeldbruck",
    7834: "Pasing",
    7835: "München",
    7836: "München-Trudering",
    7837: "Markt Schwaben",
    7838: "Albaching",
    7839: "Haag (Obb)",
    7840: "Kraiburg Inn",
    7841: "Garching (Alz)",
    7842: "Burghausen",
    7908: "Münster",
    7909: "Winzenheim",
    7910: "Colmar (Elsass)",
    7911: "Breisach",
    7912: "Eichstetten",
    7913: "Waldkirch",
    7914: "St. Peter",
    7915: "Furtwangen",
    7916: "Villingen",
    7917: "Schwenningen",
    7918: "Spaichingen",
    7919: "Nendingen",
    7920: "Leibertingen",
    7921: "Sigmaringen",
    7922: "Saulgau",
    7923: "Buchau",
    7924: "Biberach",
    7925: "Ochsenhausen",
    7926: "Erolzheim",
    7927: "Amendingen",
    7928: "Mindelheim",
    7929: "Bad Wörishofen",
    7930: "Buchloe",
    7931: "Landsberg am Lech",
    7932: "Utting (Ammersee)",
    7933: "Weßling",
    7934: "Starnberg Nord",
    7935: "München-Solln",
    7936: "Zorneding",
    7937: "Garfing (b München)",
    7938: "Steinhöring",
    7939: "Wasserburg (Inn)",
    7940: "Obing",
    7941: "Trostberg",
    7942: "Tittmoning",
    8007: "Wildenstein",
    8008: "Lautenbach",
    8009: "Gebweiler",
    8010: "Niederenzen",
    8011: "Hartheim",
    8012: "Ehrenstetten",
    8013: "Freiburg",
    8014: "Höllsteig",
    8015: "Neustadt",
    8016: "Donaueschingen",
    8017: "Geisingen",
    8018: "Tuttlingen",
    8019: "Neuhausen",
    8020: "Messkirch",
    8021: "Pfullendorf",
    8022: "Ostrach",
    8023: "Altshausen",
    8024: "Waldsee",
    8025: "Wurzach",
    8026: "Aitrach",
    8027: "Memmingen",
    8028: "Markt Rettenbach",
    8029: "Irsee",
    8030: "Waal",
    8031: "Denklingen",
    8032: "Dießen",
    8033: "Tutzing",
    8034: "Starnberg Süd",
    8035: "Sauerlach",
    8036: "Otterfing",
    8037: "Glonn",
    8038: "Rott am Inn",
    8039: "Endorf",
    8040: "Eggstätt",
    8041: "Traunreuth",
    8042: "Waging",
    8043: "Laufen",
    8107: "Urbis",
    8108: "Thann",
    8109: "Sennheim",
    8110: "Ensisheim",
    8111: "Müllheim",
    8112: "Staufen",
    8113: "Todtnau",
    8114: "Feldberg",
    8115: "Lenzkirch",
    8116: "Bonndorf",
    8117: "Blumberg",
    8118: "Engen",
    8119: "Eigeltingen",
    8120: "Stockach",
    8121: "Heiligenberg",
    8122: "Wilhelmsdorf",
    8123: "Weingarten",
    8124: "Reute",
    8125: "Diepoldshofen",
    8126: "Leutkirch",
    8127: "Grönenbach",
    8128: "Obergünzburg",
    8129: "Kaufbeuren",
    8130: "Bidingen",
    8131: "Schongau",
    8132: "Weilheim",
    8133: "Seeshaupt",
    8134: "Königsdorf",
    8135: "Sachsenkam",
    8136: "Holzkirchen",
    8137: "Bruckmühl",
    8138: "Rosenheim",
    8139: "Stephanskirchen",
    8140: "Prien am Chiemsee",
    8141: "Traunstein",
    8142: "Teisendorf",
    8143: "Freilassing",
    8207: "Nasmünster",
    8208: "Sentheim",
    8209: "Mühlhausen West",
    8210: "Mühlhausen Ost",
    8211: "Kandern",
    8212: "Wies",
    8213: "Schönau",
    8214: "St. Blasien",
    8215: "Grafenhausen",
    8216: "Stühlingen",
    8217: "Wiechs",
    8218: "Hilzingen",
    8219: "Singen",
    8220: "Überlingen",
    8221: "Mainau",
    8222: "Oberteuringen",
    8223: "Ravensburg",
    8224: "Waldburg",
    8225: "Kisslegg",
    8226: "Herlazhofen",
    8227: "Kempten",
    8228: "Wildpoldsried",
    8229: "Marktoberdorf",
    8230: "Lechbruck",
    8231: "Peiting",
    8232: "Uffing am Staffelsee",
    8233: "Iffeldorf",
    8234: "Penzberg",
    8235: "Bad Tölz",
    8236: "Tegernsee",
    8237: "Miesbach",
    8238: "Neubeuern",
    8239: "Niederaschau",
    8240: "Marquartstein",
    8241: "Ruhpolding",
    8242: "Inzell",
    8243: "Bad Reichenhall",
    8308: "Dammerkirch",
    8309: "Altkirch",
    8310: "Landser",
    8311: "Lörrach",
    8312: "Schopfheim",
    8313: "Wehr",
    8314: "Görwihl",
    8315: "Waldshut",
    8316: "Grießen",
    8317: "Jestetten",
    8318: "Gailingen",
    8319: "Öhningen",
    8320: "Reichenau",
    8321: "Konstanz",
    8322: "Friedrichshafen",
    8323: "Tettnang",
    8324: "Neukirch",
    8325: "Wangen",
    8326: "Isny",
    8327: "Buchenberg",
    8328: "Nesselwang West",
    8329: "Nesselwang",
    8330: "Roßhaupten",
    8331: "Bayersoien",
    8332: "Unterammergau",
    8333: "Murnau",
    8334: "Kochel",
    8335: "Lenggries",
    8336: "Rottach-Egern",
    8337: "Josefsthal",
    8338: "Bayrischzell",
    8339: "Oberaudorfäää",
    8340: "Reit im Winkl",
    8341: "Seegatterl",
    8342: "Schneizlreuth",
    8343: "Berchtesgaden West",
    8344: "Berchtesgaden",
    8408: "Friesen",
    8409: "Hirsingen",
    8410: "Volkensberg",
    8411: "Weil",
    8412: "Wyhlen",
    8413: "Säckingen",
    8414: "Laufenburg",
    8415: "Dangstetten",
    8416: "Lienheim",
    8417: "Eglisau",
    8423: "Langenargen",
    8424: "Lindau",
    8425: "Weiler",
    8426: "Oberstaufen",
    8427: "Immenstadt",
    8428: "Hindelang",
    8429: "Pfronten",
    8430: "Füssen",
    8431: "Linderhof",
    8432: "Oberammergau",
    8433: "Eschenlohe",
    8434: "Vorderriß",
    8435: "Fall",
    8442: "Hirschbichl",
    8443: "Königsee",
    8444: "Hoher Göll",
    8509: "Pfirt",
    8526: "Balderschwang",
    8527: "Oberstdorf",
    8528: "Hinterstein",
    8531: "Zugspitze",
    8532: "Garmisch Partenkirchen",
    8533: "Mittenwald",
    8534: "Östl. Karwendelspitze",
    8543: "Funtensee",
    8626: "Hoher Ifen",
    8627: "Einödsbach",
    8628: "Hochvogel",
    8727: "Biberkopf"
}
